(function (global, factory) {
typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports) :
typeof define === 'function' && define.amd ? define(['exports'], factory) :
(global = global || self, factory(global.predicates = {}));
}(this, function (exports) { 'use strict';

const epsilon = 1.1102230246251565e-16;
const splitter = 134217729;
const resulterrbound = (3 + 8 * epsilon) * epsilon;

function fast_expansion_sum_zeroelim(elen, e, flen, f, h) {
    let Q, Qnew, hh, bvirt;
    let enow = e[0];
    let fnow = f[0];
    let eindex = 0;
    let findex = 0;
    if ((fnow > enow) === (fnow > -enow)) {
        Q = enow;
        enow = e[++eindex];
    } else {
        Q = fnow;
        fnow = f[++findex];
    }
    let hindex = 0;
    if ((eindex < elen) && (findex < flen)) {
        if ((fnow > enow) === (fnow > -enow)) {
            Qnew = enow + Q;
            hh = Q - (Qnew - enow);
            enow = e[++eindex];
        } else {
            Qnew = fnow + Q;
            hh = Q - (Qnew - fnow);
            fnow = f[++findex];
        }
        Q = Qnew;
        if (hh !== 0) {
            h[hindex++] = hh;
        }
        while ((eindex < elen) && (findex < flen)) {
            if ((fnow > enow) === (fnow > -enow)) {
                Qnew = Q + enow;
                bvirt = Qnew - Q;
                hh = Q - (Qnew - bvirt) + (enow - bvirt);
                enow = e[++eindex];
            } else {
                Qnew = Q + fnow;
                bvirt = Qnew - Q;
                hh = Q - (Qnew - bvirt) + (fnow - bvirt);
                fnow = f[++findex];
            }
            Q = Qnew;
            if (hh !== 0) {
                h[hindex++] = hh;
            }
        }
    }
    while (eindex < elen) {
        Qnew = Q + enow;
        bvirt = Qnew - Q;
        hh = Q - (Qnew - bvirt) + (enow - bvirt);
        enow = e[++eindex];
        Q = Qnew;
        if (hh !== 0) {
            h[hindex++] = hh;
        }
    }
    while (findex < flen) {
        Qnew = Q + fnow;
        bvirt = Qnew - Q;
        hh = Q - (Qnew - bvirt) + (fnow - bvirt);
        fnow = f[++findex];
        Q = Qnew;
        if (hh !== 0) {
            h[hindex++] = hh;
        }
    }
    if ((Q !== 0) || (hindex === 0)) {
        h[hindex++] = Q;
    }
    return hindex;
}

function scale_expansion_zeroelim(elen, e, b, h) {
    let Q, sum, hh, product1, product0;
    let bvirt, c, ahi, alo, bhi, blo;

    c = splitter * b;
    bhi = c - (c - b);
    blo = b - bhi;
    Q = e[0] * b;
    c = splitter * e[0];
    ahi = c - (c - e[0]);
    alo = e[0] - ahi;
    hh = alo * blo - (Q - ahi * bhi - alo * bhi - ahi * blo);
    let hindex = 0;
    if (hh !== 0) {
        h[hindex++] = hh;
    }
    for (let eindex = 1; eindex < elen; eindex++) {
        const enow = e[eindex];
        product1 = enow * b;
        c = splitter * enow;
        ahi = c - (c - enow);
        alo = enow - ahi;
        product0 = alo * blo - (product1 - ahi * bhi - alo * bhi - ahi * blo);
        sum = Q + product0;
        bvirt = sum - Q;
        hh = Q - (sum - bvirt) + (product0 - bvirt);
        if (hh !== 0) {
            h[hindex++] = hh;
        }
        Q = product1 + sum;
        hh = sum - (Q - product1);
        if (hh !== 0) {
            h[hindex++] = hh;
        }
    }
    if ((Q !== 0.0) || (hindex === 0)) {
        h[hindex++] = Q;
    }
    return hindex;
}

function estimate(elen, e) {
    let Q = e[0];
    for (let i = 1; i < elen; i++) Q += e[i];
    return Q;
}

function vec(n) {
    return new Float64Array(n);
}

const o3derrboundA = (7 + 56 * epsilon) * epsilon;
const o3derrboundB = (3 + 28 * epsilon) * epsilon;
const o3derrboundC = (26 + 288 * epsilon) * epsilon * epsilon;

const bc = vec(4);
const ca = vec(4);
const ab = vec(4);
const adet = vec(8);
const bdet = vec(8);
const cdet = vec(8);
const abdet = vec(16);
const fin1 = vec(192);
const fin2 = vec(192);
const at_b = vec(4);
const at_c = vec(4);
const bt_c = vec(4);
const bt_a = vec(4);
const ct_a = vec(4);
const ct_b = vec(4);
const bct = vec(8);
const cat = vec(8);
const abt = vec(8);
const u = vec(4);
const v = vec(12);
const w = vec(16);

function orient3dadapt(ax, ay, az, bx, by, bz, cx, cy, cz, dx, dy, dz, permanent) {
    let bdxcdy1, cdxbdy1, cdxady1, adxcdy1, adxbdy1, bdxady1;
    let bdxcdy0, cdxbdy0, cdxady0, adxcdy0, adxbdy0, bdxady0;
    let bc3, ca3, ab3;
    let finnow, finother, finswap;
    let finlength;

    let adxtail, bdxtail, cdxtail;
    let adytail, bdytail, cdytail;
    let adztail, bdztail, cdztail;
    let at_blarge, at_clarge;
    let bt_clarge, bt_alarge;
    let ct_alarge, ct_blarge;
    let at_blen, at_clen, bt_clen, bt_alen, ct_alen, ct_blen;
    let bdxt_cdy1, cdxt_bdy1, cdxt_ady1;
    let adxt_cdy1, adxt_bdy1, bdxt_ady1;
    let bdxt_cdy0, cdxt_bdy0, cdxt_ady0;
    let adxt_cdy0, adxt_bdy0, bdxt_ady0;
    let bdyt_cdx1, cdyt_bdx1, cdyt_adx1;
    let adyt_cdx1, adyt_bdx1, bdyt_adx1;
    let bdyt_cdx0, cdyt_bdx0, cdyt_adx0;
    let adyt_cdx0, adyt_bdx0, bdyt_adx0;
    let bdxt_cdyt1, cdxt_bdyt1, cdxt_adyt1;
    let adxt_cdyt1, adxt_bdyt1, bdxt_adyt1;
    let bdxt_cdyt0, cdxt_bdyt0, cdxt_adyt0;
    let adxt_cdyt0, adxt_bdyt0, bdxt_adyt0;
    let u3;
    let vlength, wlength;
    let negate;

    let bvirt, c, ahi, alo, bhi, blo, _i, _j, _k, _0;

    const adx = ax - dx;
    const bdx = bx - dx;
    const cdx = cx - dx;
    const ady = ay - dy;
    const bdy = by - dy;
    const cdy = cy - dy;
    const adz = az - dz;
    const bdz = bz - dz;
    const cdz = cz - dz;

    bdxcdy1 = bdx * cdy;
    c = splitter * bdx;
    ahi = c - (c - bdx);
    alo = bdx - ahi;
    c = splitter * cdy;
    bhi = c - (c - cdy);
    blo = cdy - bhi;
    bdxcdy0 = alo * blo - (bdxcdy1 - ahi * bhi - alo * bhi - ahi * blo);
    cdxbdy1 = cdx * bdy;
    c = splitter * cdx;
    ahi = c - (c - cdx);
    alo = cdx - ahi;
    c = splitter * bdy;
    bhi = c - (c - bdy);
    blo = bdy - bhi;
    cdxbdy0 = alo * blo - (cdxbdy1 - ahi * bhi - alo * bhi - ahi * blo);
    _i = bdxcdy0 - cdxbdy0;
    bvirt = bdxcdy0 - _i;
    bc[0] = bdxcdy0 - (_i + bvirt) + (bvirt - cdxbdy0);
    _j = bdxcdy1 + _i;
    bvirt = _j - bdxcdy1;
    _0 = bdxcdy1 - (_j - bvirt) + (_i - bvirt);
    _i = _0 - cdxbdy1;
    bvirt = _0 - _i;
    bc[1] = _0 - (_i + bvirt) + (bvirt - cdxbdy1);
    bc3 = _j + _i;
    bvirt = bc3 - _j;
    bc[2] = _j - (bc3 - bvirt) + (_i - bvirt);
    bc[3] = bc3;
    const alen = scale_expansion_zeroelim(4, bc, adz, adet);

    cdxady1 = cdx * ady;
    c = splitter * cdx;
    ahi = c - (c - cdx);
    alo = cdx - ahi;
    c = splitter * ady;
    bhi = c - (c - ady);
    blo = ady - bhi;
    cdxady0 = alo * blo - (cdxady1 - ahi * bhi - alo * bhi - ahi * blo);
    adxcdy1 = adx * cdy;
    c = splitter * adx;
    ahi = c - (c - adx);
    alo = adx - ahi;
    c = splitter * cdy;
    bhi = c - (c - cdy);
    blo = cdy - bhi;
    adxcdy0 = alo * blo - (adxcdy1 - ahi * bhi - alo * bhi - ahi * blo);
    _i = cdxady0 - adxcdy0;
    bvirt = cdxady0 - _i;
    ca[0] = cdxady0 - (_i + bvirt) + (bvirt - adxcdy0);
    _j = cdxady1 + _i;
    bvirt = _j - cdxady1;
    _0 = cdxady1 - (_j - bvirt) + (_i - bvirt);
    _i = _0 - adxcdy1;
    bvirt = _0 - _i;
    ca[1] = _0 - (_i + bvirt) + (bvirt - adxcdy1);
    ca3 = _j + _i;
    bvirt = ca3 - _j;
    ca[2] = _j - (ca3 - bvirt) + (_i - bvirt);
    ca[3] = ca3;
    const blen = scale_expansion_zeroelim(4, ca, bdz, bdet);

    adxbdy1 = adx * bdy;
    c = splitter * adx;
    ahi = c - (c - adx);
    alo = adx - ahi;
    c = splitter * bdy;
    bhi = c - (c - bdy);
    blo = bdy - bhi;
    adxbdy0 = alo * blo - (adxbdy1 - ahi * bhi - alo * bhi - ahi * blo);
    bdxady1 = bdx * ady;
    c = splitter * bdx;
    ahi = c - (c - bdx);
    alo = bdx - ahi;
    c = splitter * ady;
    bhi = c - (c - ady);
    blo = ady - bhi;
    bdxady0 = alo * blo - (bdxady1 - ahi * bhi - alo * bhi - ahi * blo);
    _i = adxbdy0 - bdxady0;
    bvirt = adxbdy0 - _i;
    ab[0] = adxbdy0 - (_i + bvirt) + (bvirt - bdxady0);
    _j = adxbdy1 + _i;
    bvirt = _j - adxbdy1;
    _0 = adxbdy1 - (_j - bvirt) + (_i - bvirt);
    _i = _0 - bdxady1;
    bvirt = _0 - _i;
    ab[1] = _0 - (_i + bvirt) + (bvirt - bdxady1);
    ab3 = _j + _i;
    bvirt = ab3 - _j;
    ab[2] = _j - (ab3 - bvirt) + (_i - bvirt);
    ab[3] = ab3;
    const clen = scale_expansion_zeroelim(4, ab, cdz, cdet);

    const ablen = fast_expansion_sum_zeroelim(alen, adet, blen, bdet, abdet);
    finlength = fast_expansion_sum_zeroelim(ablen, abdet, clen, cdet, fin1);

    let det = estimate(finlength, fin1);
    let errbound = o3derrboundB * permanent;
    if (det >= errbound || -det >= errbound) {
        return det;
    }

    bvirt = ax - adx;
    adxtail = ax - (adx + bvirt) + (bvirt - dx);
    bvirt = bx - bdx;
    bdxtail = bx - (bdx + bvirt) + (bvirt - dx);
    bvirt = cx - cdx;
    cdxtail = cx - (cdx + bvirt) + (bvirt - dx);
    bvirt = ay - ady;
    adytail = ay - (ady + bvirt) + (bvirt - dy);
    bvirt = by - bdy;
    bdytail = by - (bdy + bvirt) + (bvirt - dy);
    bvirt = cy - cdy;
    cdytail = cy - (cdy + bvirt) + (bvirt - dy);
    bvirt = az - adz;
    adztail = az - (adz + bvirt) + (bvirt - dz);
    bvirt = bz - bdz;
    bdztail = bz - (bdz + bvirt) + (bvirt - dz);
    bvirt = cz - cdz;
    cdztail = cz - (cdz + bvirt) + (bvirt - dz);

    if (adxtail === 0 && bdxtail === 0 && cdxtail === 0 &&
        adytail === 0 && bdytail === 0 && cdytail === 0 &&
        adztail === 0 && bdztail === 0 && cdztail === 0) {
        return det;
    }

    errbound = o3derrboundC * permanent + resulterrbound * Math.abs(det);
    det +=
        adz * (bdx * cdytail + cdy * bdxtail - (bdy * cdxtail + cdx * bdytail)) + adztail * (bdx * cdy - bdy * cdx) +
        bdz * (cdx * adytail + ady * cdxtail - (cdy * adxtail + adx * cdytail)) + bdztail * (cdx * ady - cdy * adx) +
        cdz * (adx * bdytail + bdy * adxtail - (ady * bdxtail + bdx * adytail)) + cdztail * (adx * bdy - ady * bdx);
    if (det >= errbound || -det >= errbound) {
        return det;
    }

    finnow = fin1;
    finother = fin2;

    if (adxtail === 0) {
        if (adytail === 0) {
            at_b[0] = 0;
            at_blen = 1;
            at_c[0] = 0;
            at_clen = 1;
        } else {
            negate = -adytail;
            at_blarge = negate * bdx;
            c = splitter * negate;
            ahi = c - (c - negate);
            alo = negate - ahi;
            c = splitter * bdx;
            bhi = c - (c - bdx);
            blo = bdx - bhi;
            at_b[0] = alo * blo - (at_blarge - ahi * bhi - alo * bhi - ahi * blo);
            at_b[1] = at_blarge;
            at_blen = 2;
            at_clarge = adytail * cdx;
            c = splitter * adytail;
            ahi = c - (c - adytail);
            alo = adytail - ahi;
            c = splitter * cdx;
            bhi = c - (c - cdx);
            blo = cdx - bhi;
            at_c[0] = alo * blo - (at_clarge - ahi * bhi - alo * bhi - ahi * blo);
            at_c[1] = at_clarge;
            at_clen = 2;
        }
    } else {
        if (adytail === 0) {
            at_blarge = adxtail * bdy;
            c = splitter * adxtail;
            ahi = c - (c - adxtail);
            alo = adxtail - ahi;
            c = splitter * bdy;
            bhi = c - (c - bdy);
            blo = bdy - bhi;
            at_b[0] = alo * blo - (at_blarge - ahi * bhi - alo * bhi - ahi * blo);
            at_b[1] = at_blarge;
            at_blen = 2;
            negate = -adxtail;
            at_clarge = negate * cdy;
            c = splitter * negate;
            ahi = c - (c - negate);
            alo = negate - ahi;
            c = splitter * cdy;
            bhi = c - (c - cdy);
            blo = cdy - bhi;
            at_c[0] = alo * blo - (at_clarge - ahi * bhi - alo * bhi - ahi * blo);
            at_c[1] = at_clarge;
            at_clen = 2;
        } else {
            adxt_bdy1 = adxtail * bdy;
            c = splitter * adxtail;
            ahi = c - (c - adxtail);
            alo = adxtail - ahi;
            c = splitter * bdy;
            bhi = c - (c - bdy);
            blo = bdy - bhi;
            adxt_bdy0 = alo * blo - (adxt_bdy1 - ahi * bhi - alo * bhi - ahi * blo);
            adyt_bdx1 = adytail * bdx;
            c = splitter * adytail;
            ahi = c - (c - adytail);
            alo = adytail - ahi;
            c = splitter * bdx;
            bhi = c - (c - bdx);
            blo = bdx - bhi;
            adyt_bdx0 = alo * blo - (adyt_bdx1 - ahi * bhi - alo * bhi - ahi * blo);
            _i = adxt_bdy0 - adyt_bdx0;
            bvirt = adxt_bdy0 - _i;
            at_b[0] = adxt_bdy0 - (_i + bvirt) + (bvirt - adyt_bdx0);
            _j = adxt_bdy1 + _i;
            bvirt = _j - adxt_bdy1;
            _0 = adxt_bdy1 - (_j - bvirt) + (_i - bvirt);
            _i = _0 - adyt_bdx1;
            bvirt = _0 - _i;
            at_b[1] = _0 - (_i + bvirt) + (bvirt - adyt_bdx1);
            at_blarge = _j + _i;
            bvirt = at_blarge - _j;
            at_b[2] = _j - (at_blarge - bvirt) + (_i - bvirt);
            at_b[3] = at_blarge;
            at_blen = 4;
            adyt_cdx1 = adytail * cdx;
            c = splitter * adytail;
            ahi = c - (c - adytail);
            alo = adytail - ahi;
            c = splitter * cdx;
            bhi = c - (c - cdx);
            blo = cdx - bhi;
            adyt_cdx0 = alo * blo - (adyt_cdx1 - ahi * bhi - alo * bhi - ahi * blo);
            adxt_cdy1 = adxtail * cdy;
            c = splitter * adxtail;
            ahi = c - (c - adxtail);
            alo = adxtail - ahi;
            c = splitter * cdy;
            bhi = c - (c - cdy);
            blo = cdy - bhi;
            adxt_cdy0 = alo * blo - (adxt_cdy1 - ahi * bhi - alo * bhi - ahi * blo);
            _i = adyt_cdx0 - adxt_cdy0;
            bvirt = adyt_cdx0 - _i;
            at_c[0] = adyt_cdx0 - (_i + bvirt) + (bvirt - adxt_cdy0);
            _j = adyt_cdx1 + _i;
            bvirt = _j - adyt_cdx1;
            _0 = adyt_cdx1 - (_j - bvirt) + (_i - bvirt);
            _i = _0 - adxt_cdy1;
            bvirt = _0 - _i;
            at_c[1] = _0 - (_i + bvirt) + (bvirt - adxt_cdy1);
            at_clarge = _j + _i;
            bvirt = at_clarge - _j;
            at_c[2] = _j - (at_clarge - bvirt) + (_i - bvirt);
            at_c[3] = at_clarge;
            at_clen = 4;
        }
    }
    if (bdxtail === 0) {
        if (bdytail === 0) {
            bt_c[0] = 0;
            bt_clen = 1;
            bt_a[0] = 0;
            bt_alen = 1;
        } else {
            negate = -bdytail;
            bt_clarge = negate * cdx;
            c = splitter * negate;
            ahi = c - (c - negate);
            alo = negate - ahi;
            c = splitter * cdx;
            bhi = c - (c - cdx);
            blo = cdx - bhi;
            bt_c[0] = alo * blo - (bt_clarge - ahi * bhi - alo * bhi - ahi * blo);
            bt_c[1] = bt_clarge;
            bt_clen = 2;
            bt_alarge = bdytail * adx;
            c = splitter * bdytail;
            ahi = c - (c - bdytail);
            alo = bdytail - ahi;
            c = splitter * adx;
            bhi = c - (c - adx);
            blo = adx - bhi;
            bt_a[0] = alo * blo - (bt_alarge - ahi * bhi - alo * bhi - ahi * blo);
            bt_a[1] = bt_alarge;
            bt_alen = 2;
        }
    } else {
        if (bdytail === 0) {
            bt_clarge = bdxtail * cdy;
            c = splitter * bdxtail;
            ahi = c - (c - bdxtail);
            alo = bdxtail - ahi;
            c = splitter * cdy;
            bhi = c - (c - cdy);
            blo = cdy - bhi;
            bt_c[0] = alo * blo - (bt_clarge - ahi * bhi - alo * bhi - ahi * blo);
            bt_c[1] = bt_clarge;
            bt_clen = 2;
            negate = -bdxtail;
            bt_alarge = negate * ady;
            c = splitter * negate;
            ahi = c - (c - negate);
            alo = negate - ahi;
            c = splitter * ady;
            bhi = c - (c - ady);
            blo = ady - bhi;
            bt_a[0] = alo * blo - (bt_alarge - ahi * bhi - alo * bhi - ahi * blo);
            bt_a[1] = bt_alarge;
            bt_alen = 2;
        } else {
            bdxt_cdy1 = bdxtail * cdy;
            c = splitter * bdxtail;
            ahi = c - (c - bdxtail);
            alo = bdxtail - ahi;
            c = splitter * cdy;
            bhi = c - (c - cdy);
            blo = cdy - bhi;
            bdxt_cdy0 = alo * blo - (bdxt_cdy1 - ahi * bhi - alo * bhi - ahi * blo);
            bdyt_cdx1 = bdytail * cdx;
            c = splitter * bdytail;
            ahi = c - (c - bdytail);
            alo = bdytail - ahi;
            c = splitter * cdx;
            bhi = c - (c - cdx);
            blo = cdx - bhi;
            bdyt_cdx0 = alo * blo - (bdyt_cdx1 - ahi * bhi - alo * bhi - ahi * blo);
            _i = bdxt_cdy0 - bdyt_cdx0;
            bvirt = bdxt_cdy0 - _i;
            bt_c[0] = bdxt_cdy0 - (_i + bvirt) + (bvirt - bdyt_cdx0);
            _j = bdxt_cdy1 + _i;
            bvirt = _j - bdxt_cdy1;
            _0 = bdxt_cdy1 - (_j - bvirt) + (_i - bvirt);
            _i = _0 - bdyt_cdx1;
            bvirt = _0 - _i;
            bt_c[1] = _0 - (_i + bvirt) + (bvirt - bdyt_cdx1);
            bt_clarge = _j + _i;
            bvirt = bt_clarge - _j;
            bt_c[2] = _j - (bt_clarge - bvirt) + (_i - bvirt);
            bt_c[3] = bt_clarge;
            bt_clen = 4;
            bdyt_adx1 = bdytail * adx;
            c = splitter * bdytail;
            ahi = c - (c - bdytail);
            alo = bdytail - ahi;
            c = splitter * adx;
            bhi = c - (c - adx);
            blo = adx - bhi;
            bdyt_adx0 = alo * blo - (bdyt_adx1 - ahi * bhi - alo * bhi - ahi * blo);
            bdxt_ady1 = bdxtail * ady;
            c = splitter * bdxtail;
            ahi = c - (c - bdxtail);
            alo = bdxtail - ahi;
            c = splitter * ady;
            bhi = c - (c - ady);
            blo = ady - bhi;
            bdxt_ady0 = alo * blo - (bdxt_ady1 - ahi * bhi - alo * bhi - ahi * blo);
            _i = bdyt_adx0 - bdxt_ady0;
            bvirt = bdyt_adx0 - _i;
            bt_a[0] = bdyt_adx0 - (_i + bvirt) + (bvirt - bdxt_ady0);
            _j = bdyt_adx1 + _i;
            bvirt = _j - bdyt_adx1;
            _0 = bdyt_adx1 - (_j - bvirt) + (_i - bvirt);
            _i = _0 - bdxt_ady1;
            bvirt = _0 - _i;
            bt_a[1] = _0 - (_i + bvirt) + (bvirt - bdxt_ady1);
            bt_alarge = _j + _i;
            bvirt = bt_alarge - _j;
            bt_a[2] = _j - (bt_alarge - bvirt) + (_i - bvirt);
            bt_a[3] = bt_alarge;
            bt_alen = 4;
        }
    }
    if (cdxtail === 0) {
        if (cdytail === 0) {
            ct_a[0] = 0;
            ct_alen = 1;
            ct_b[0] = 0;
            ct_blen = 1;
        } else {
            negate = -cdytail;
            ct_alarge = negate * adx;
            c = splitter * negate;
            ahi = c - (c - negate);
            alo = negate - ahi;
            c = splitter * adx;
            bhi = c - (c - adx);
            blo = adx - bhi;
            ct_a[0] = alo * blo - (ct_alarge - ahi * bhi - alo * bhi - ahi * blo);
            ct_a[1] = ct_alarge;
            ct_alen = 2;
            ct_blarge = cdytail * bdx;
            c = splitter * cdytail;
            ahi = c - (c - cdytail);
            alo = cdytail - ahi;
            c = splitter * bdx;
            bhi = c - (c - bdx);
            blo = bdx - bhi;
            ct_b[0] = alo * blo - (ct_blarge - ahi * bhi - alo * bhi - ahi * blo);
            ct_b[1] = ct_blarge;
            ct_blen = 2;
        }
    } else {
        if (cdytail === 0) {
            ct_alarge = cdxtail * ady;
            c = splitter * cdxtail;
            ahi = c - (c - cdxtail);
            alo = cdxtail - ahi;
            c = splitter * ady;
            bhi = c - (c - ady);
            blo = ady - bhi;
            ct_a[0] = alo * blo - (ct_alarge - ahi * bhi - alo * bhi - ahi * blo);
            ct_a[1] = ct_alarge;
            ct_alen = 2;
            negate = -cdxtail;
            ct_blarge = negate * bdy;
            c = splitter * negate;
            ahi = c - (c - negate);
            alo = negate - ahi;
            c = splitter * bdy;
            bhi = c - (c - bdy);
            blo = bdy - bhi;
            ct_b[0] = alo * blo - (ct_blarge - ahi * bhi - alo * bhi - ahi * blo);
            ct_b[1] = ct_blarge;
            ct_blen = 2;
        } else {
            cdxt_ady1 = cdxtail * ady;
            c = splitter * cdxtail;
            ahi = c - (c - cdxtail);
            alo = cdxtail - ahi;
            c = splitter * ady;
            bhi = c - (c - ady);
            blo = ady - bhi;
            cdxt_ady0 = alo * blo - (cdxt_ady1 - ahi * bhi - alo * bhi - ahi * blo);
            cdyt_adx1 = cdytail * adx;
            c = splitter * cdytail;
            ahi = c - (c - cdytail);
            alo = cdytail - ahi;
            c = splitter * adx;
            bhi = c - (c - adx);
            blo = adx - bhi;
            cdyt_adx0 = alo * blo - (cdyt_adx1 - ahi * bhi - alo * bhi - ahi * blo);
            _i = cdxt_ady0 - cdyt_adx0;
            bvirt = cdxt_ady0 - _i;
            ct_a[0] = cdxt_ady0 - (_i + bvirt) + (bvirt - cdyt_adx0);
            _j = cdxt_ady1 + _i;
            bvirt = _j - cdxt_ady1;
            _0 = cdxt_ady1 - (_j - bvirt) + (_i - bvirt);
            _i = _0 - cdyt_adx1;
            bvirt = _0 - _i;
            ct_a[1] = _0 - (_i + bvirt) + (bvirt - cdyt_adx1);
            ct_alarge = _j + _i;
            bvirt = ct_alarge - _j;
            ct_a[2] = _j - (ct_alarge - bvirt) + (_i - bvirt);
            ct_a[3] = ct_alarge;
            ct_alen = 4;
            cdyt_bdx1 = cdytail * bdx;
            c = splitter * cdytail;
            ahi = c - (c - cdytail);
            alo = cdytail - ahi;
            c = splitter * bdx;
            bhi = c - (c - bdx);
            blo = bdx - bhi;
            cdyt_bdx0 = alo * blo - (cdyt_bdx1 - ahi * bhi - alo * bhi - ahi * blo);
            cdxt_bdy1 = cdxtail * bdy;
            c = splitter * cdxtail;
            ahi = c - (c - cdxtail);
            alo = cdxtail - ahi;
            c = splitter * bdy;
            bhi = c - (c - bdy);
            blo = bdy - bhi;
            cdxt_bdy0 = alo * blo - (cdxt_bdy1 - ahi * bhi - alo * bhi - ahi * blo);
            _i = cdyt_bdx0 - cdxt_bdy0;
            bvirt = cdyt_bdx0 - _i;
            ct_b[0] = cdyt_bdx0 - (_i + bvirt) + (bvirt - cdxt_bdy0);
            _j = cdyt_bdx1 + _i;
            bvirt = _j - cdyt_bdx1;
            _0 = cdyt_bdx1 - (_j - bvirt) + (_i - bvirt);
            _i = _0 - cdxt_bdy1;
            bvirt = _0 - _i;
            ct_b[1] = _0 - (_i + bvirt) + (bvirt - cdxt_bdy1);
            ct_blarge = _j + _i;
            bvirt = ct_blarge - _j;
            ct_b[2] = _j - (ct_blarge - bvirt) + (_i - bvirt);
            ct_b[3] = ct_blarge;
            ct_blen = 4;
        }
    }

    const bctlen = fast_expansion_sum_zeroelim(bt_clen, bt_c, ct_blen, ct_b, bct);
    wlength = scale_expansion_zeroelim(bctlen, bct, adz, w);
    finlength = fast_expansion_sum_zeroelim(finlength, finnow, wlength, w, finother);
    finswap = finnow; finnow = finother; finother = finswap;

    const catlen = fast_expansion_sum_zeroelim(ct_alen, ct_a, at_clen, at_c, cat);
    wlength = scale_expansion_zeroelim(catlen, cat, bdz, w);
    finlength = fast_expansion_sum_zeroelim(finlength, finnow, wlength, w, finother);
    finswap = finnow; finnow = finother; finother = finswap;

    const abtlen = fast_expansion_sum_zeroelim(at_blen, at_b, bt_alen, bt_a, abt);
    wlength = scale_expansion_zeroelim(abtlen, abt, cdz, w);
    finlength = fast_expansion_sum_zeroelim(finlength, finnow, wlength, w, finother);
    finswap = finnow; finnow = finother; finother = finswap;

    if (adztail !== 0) {
        vlength = scale_expansion_zeroelim(4, bc, adztail, v);
        finlength = fast_expansion_sum_zeroelim(finlength, finnow, vlength, v, finother);
        finswap = finnow; finnow = finother; finother = finswap;
    }
    if (bdztail !== 0) {
        vlength = scale_expansion_zeroelim(4, ca, bdztail, v);
        finlength = fast_expansion_sum_zeroelim(finlength, finnow, vlength, v, finother);
        finswap = finnow; finnow = finother; finother = finswap;
    }
    if (cdztail !== 0) {
        vlength = scale_expansion_zeroelim(4, ab, cdztail, v);
        finlength = fast_expansion_sum_zeroelim(finlength, finnow, vlength, v, finother);
        finswap = finnow; finnow = finother; finother = finswap;
    }

    if (adxtail !== 0) {
        if (bdytail !== 0) {
            adxt_bdyt1 = adxtail * bdytail;
            c = splitter * adxtail;
            ahi = c - (c - adxtail);
            alo = adxtail - ahi;
            c = splitter * bdytail;
            bhi = c - (c - bdytail);
            blo = bdytail - bhi;
            adxt_bdyt0 = alo * blo - (adxt_bdyt1 - ahi * bhi - alo * bhi - ahi * blo);
            c = splitter * cdz;
            bhi = c - (c - cdz);
            blo = cdz - bhi;
            _i = adxt_bdyt0 * cdz;
            c = splitter * adxt_bdyt0;
            ahi = c - (c - adxt_bdyt0);
            alo = adxt_bdyt0 - ahi;
            u[0] = alo * blo - (_i - ahi * bhi - alo * bhi - ahi * blo);
            _j = adxt_bdyt1 * cdz;
            c = splitter * adxt_bdyt1;
            ahi = c - (c - adxt_bdyt1);
            alo = adxt_bdyt1 - ahi;
            _0 = alo * blo - (_j - ahi * bhi - alo * bhi - ahi * blo);
            _k = _i + _0;
            bvirt = _k - _i;
            u[1] = _i - (_k - bvirt) + (_0 - bvirt);
            u3 = _j + _k;
            u[2] = _k - (u3 - _j);
            u[3] = u3;
            finlength = fast_expansion_sum_zeroelim(finlength, finnow, 4, u, finother);
            finswap = finnow; finnow = finother; finother = finswap;
            if (cdztail !== 0) {
                c = splitter * cdztail;
                bhi = c - (c - cdztail);
                blo = cdztail - bhi;
                _i = adxt_bdyt0 * cdztail;
                c = splitter * adxt_bdyt0;
                ahi = c - (c - adxt_bdyt0);
                alo = adxt_bdyt0 - ahi;
                u[0] = alo * blo - (_i - ahi * bhi - alo * bhi - ahi * blo);
                _j = adxt_bdyt1 * cdztail;
                c = splitter * adxt_bdyt1;
                ahi = c - (c - adxt_bdyt1);
                alo = adxt_bdyt1 - ahi;
                _0 = alo * blo - (_j - ahi * bhi - alo * bhi - ahi * blo);
                _k = _i + _0;
                bvirt = _k - _i;
                u[1] = _i - (_k - bvirt) + (_0 - bvirt);
                u3 = _j + _k;
                u[2] = _k - (u3 - _j);
                u[3] = u3;
                finlength = fast_expansion_sum_zeroelim(finlength, finnow, 4, u, finother);
                finswap = finnow; finnow = finother; finother = finswap;
            }
        }
        if (cdytail !== 0) {
            negate = -adxtail;
            adxt_cdyt1 = negate * cdytail;
            c = splitter * negate;
            ahi = c - (c - negate);
            alo = negate - ahi;
            c = splitter * cdytail;
            bhi = c - (c - cdytail);
            blo = cdytail - bhi;
            adxt_cdyt0 = alo * blo - (adxt_cdyt1 - ahi * bhi - alo * bhi - ahi * blo);
            c = splitter * bdz;
            bhi = c - (c - bdz);
            blo = bdz - bhi;
            _i = adxt_cdyt0 * bdz;
            c = splitter * adxt_cdyt0;
            ahi = c - (c - adxt_cdyt0);
            alo = adxt_cdyt0 - ahi;
            u[0] = alo * blo - (_i - ahi * bhi - alo * bhi - ahi * blo);
            _j = adxt_cdyt1 * bdz;
            c = splitter * adxt_cdyt1;
            ahi = c - (c - adxt_cdyt1);
            alo = adxt_cdyt1 - ahi;
            _0 = alo * blo - (_j - ahi * bhi - alo * bhi - ahi * blo);
            _k = _i + _0;
            bvirt = _k - _i;
            u[1] = _i - (_k - bvirt) + (_0 - bvirt);
            u3 = _j + _k;
            u[2] = _k - (u3 - _j);
            u[3] = u3;
            finlength = fast_expansion_sum_zeroelim(finlength, finnow, 4, u, finother);
            finswap = finnow; finnow = finother; finother = finswap;
            if (bdztail !== 0) {
                c = splitter * bdztail;
                bhi = c - (c - bdztail);
                blo = bdztail - bhi;
                _i = adxt_cdyt0 * bdztail;
                c = splitter * adxt_cdyt0;
                ahi = c - (c - adxt_cdyt0);
                alo = adxt_cdyt0 - ahi;
                u[0] = alo * blo - (_i - ahi * bhi - alo * bhi - ahi * blo);
                _j = adxt_cdyt1 * bdztail;
                c = splitter * adxt_cdyt1;
                ahi = c - (c - adxt_cdyt1);
                alo = adxt_cdyt1 - ahi;
                _0 = alo * blo - (_j - ahi * bhi - alo * bhi - ahi * blo);
                _k = _i + _0;
                bvirt = _k - _i;
                u[1] = _i - (_k - bvirt) + (_0 - bvirt);
                u3 = _j + _k;
                u[2] = _k - (u3 - _j);
                u[3] = u3;
                finlength = fast_expansion_sum_zeroelim(finlength, finnow, 4, u, finother);
                finswap = finnow; finnow = finother; finother = finswap;
            }
        }
    }
    if (bdxtail !== 0) {
        if (cdytail !== 0) {
            bdxt_cdyt1 = bdxtail * cdytail;
            c = splitter * bdxtail;
            ahi = c - (c - bdxtail);
            alo = bdxtail - ahi;
            c = splitter * cdytail;
            bhi = c - (c - cdytail);
            blo = cdytail - bhi;
            bdxt_cdyt0 = alo * blo - (bdxt_cdyt1 - ahi * bhi - alo * bhi - ahi * blo);
            c = splitter * adz;
            bhi = c - (c - adz);
            blo = adz - bhi;
            _i = bdxt_cdyt0 * adz;
            c = splitter * bdxt_cdyt0;
            ahi = c - (c - bdxt_cdyt0);
            alo = bdxt_cdyt0 - ahi;
            u[0] = alo * blo - (_i - ahi * bhi - alo * bhi - ahi * blo);
            _j = bdxt_cdyt1 * adz;
            c = splitter * bdxt_cdyt1;
            ahi = c - (c - bdxt_cdyt1);
            alo = bdxt_cdyt1 - ahi;
            _0 = alo * blo - (_j - ahi * bhi - alo * bhi - ahi * blo);
            _k = _i + _0;
            bvirt = _k - _i;
            u[1] = _i - (_k - bvirt) + (_0 - bvirt);
            u3 = _j + _k;
            u[2] = _k - (u3 - _j);
            u[3] = u3;
            finlength = fast_expansion_sum_zeroelim(finlength, finnow, 4, u, finother);
            finswap = finnow; finnow = finother; finother = finswap;
            if (adztail !== 0) {
                c = splitter * adztail;
                bhi = c - (c - adztail);
                blo = adztail - bhi;
                _i = bdxt_cdyt0 * adztail;
                c = splitter * bdxt_cdyt0;
                ahi = c - (c - bdxt_cdyt0);
                alo = bdxt_cdyt0 - ahi;
                u[0] = alo * blo - (_i - ahi * bhi - alo * bhi - ahi * blo);
                _j = bdxt_cdyt1 * adztail;
                c = splitter * bdxt_cdyt1;
                ahi = c - (c - bdxt_cdyt1);
                alo = bdxt_cdyt1 - ahi;
                _0 = alo * blo - (_j - ahi * bhi - alo * bhi - ahi * blo);
                _k = _i + _0;
                bvirt = _k - _i;
                u[1] = _i - (_k - bvirt) + (_0 - bvirt);
                u3 = _j + _k;
                u[2] = _k - (u3 - _j);
                u[3] = u3;
                finlength = fast_expansion_sum_zeroelim(finlength, finnow, 4, u, finother);
                finswap = finnow; finnow = finother; finother = finswap;
            }
        }
        if (adytail !== 0) {
            negate = -bdxtail;
            bdxt_adyt1 = negate * adytail;
            c = splitter * negate;
            ahi = c - (c - negate);
            alo = negate - ahi;
            c = splitter * adytail;
            bhi = c - (c - adytail);
            blo = adytail - bhi;
            bdxt_adyt0 = alo * blo - (bdxt_adyt1 - ahi * bhi - alo * bhi - ahi * blo);
            c = splitter * cdz;
            bhi = c - (c - cdz);
            blo = cdz - bhi;
            _i = bdxt_adyt0 * cdz;
            c = splitter * bdxt_adyt0;
            ahi = c - (c - bdxt_adyt0);
            alo = bdxt_adyt0 - ahi;
            u[0] = alo * blo - (_i - ahi * bhi - alo * bhi - ahi * blo);
            _j = bdxt_adyt1 * cdz;
            c = splitter * bdxt_adyt1;
            ahi = c - (c - bdxt_adyt1);
            alo = bdxt_adyt1 - ahi;
            _0 = alo * blo - (_j - ahi * bhi - alo * bhi - ahi * blo);
            _k = _i + _0;
            bvirt = _k - _i;
            u[1] = _i - (_k - bvirt) + (_0 - bvirt);
            u3 = _j + _k;
            u[2] = _k - (u3 - _j);
            u[3] = u3;
            finlength = fast_expansion_sum_zeroelim(finlength, finnow, 4, u, finother);
            finswap = finnow; finnow = finother; finother = finswap;
            if (cdztail !== 0) {
                c = splitter * cdztail;
                bhi = c - (c - cdztail);
                blo = cdztail - bhi;
                _i = bdxt_adyt0 * cdztail;
                c = splitter * bdxt_adyt0;
                ahi = c - (c - bdxt_adyt0);
                alo = bdxt_adyt0 - ahi;
                u[0] = alo * blo - (_i - ahi * bhi - alo * bhi - ahi * blo);
                _j = bdxt_adyt1 * cdztail;
                c = splitter * bdxt_adyt1;
                ahi = c - (c - bdxt_adyt1);
                alo = bdxt_adyt1 - ahi;
                _0 = alo * blo - (_j - ahi * bhi - alo * bhi - ahi * blo);
                _k = _i + _0;
                bvirt = _k - _i;
                u[1] = _i - (_k - bvirt) + (_0 - bvirt);
                u3 = _j + _k;
                u[2] = _k - (u3 - _j);
                u[3] = u3;
                finlength = fast_expansion_sum_zeroelim(finlength, finnow, 4, u, finother);
                finswap = finnow; finnow = finother; finother = finswap;
            }
        }
    }
    if (cdxtail !== 0) {
        if (adytail !== 0) {
            cdxt_adyt1 = cdxtail * adytail;
            c = splitter * cdxtail;
            ahi = c - (c - cdxtail);
            alo = cdxtail - ahi;
            c = splitter * adytail;
            bhi = c - (c - adytail);
            blo = adytail - bhi;
            cdxt_adyt0 = alo * blo - (cdxt_adyt1 - ahi * bhi - alo * bhi - ahi * blo);
            c = splitter * bdz;
            bhi = c - (c - bdz);
            blo = bdz - bhi;
            _i = cdxt_adyt0 * bdz;
            c = splitter * cdxt_adyt0;
            ahi = c - (c - cdxt_adyt0);
            alo = cdxt_adyt0 - ahi;
            u[0] = alo * blo - (_i - ahi * bhi - alo * bhi - ahi * blo);
            _j = cdxt_adyt1 * bdz;
            c = splitter * cdxt_adyt1;
            ahi = c - (c - cdxt_adyt1);
            alo = cdxt_adyt1 - ahi;
            _0 = alo * blo - (_j - ahi * bhi - alo * bhi - ahi * blo);
            _k = _i + _0;
            bvirt = _k - _i;
            u[1] = _i - (_k - bvirt) + (_0 - bvirt);
            u3 = _j + _k;
            u[2] = _k - (u3 - _j);
            u[3] = u3;
            finlength = fast_expansion_sum_zeroelim(finlength, finnow, 4, u, finother);
            finswap = finnow; finnow = finother; finother = finswap;
            if (bdztail !== 0) {
                c = splitter * bdztail;
                bhi = c - (c - bdztail);
                blo = bdztail - bhi;
                _i = cdxt_adyt0 * bdztail;
                c = splitter * cdxt_adyt0;
                ahi = c - (c - cdxt_adyt0);
                alo = cdxt_adyt0 - ahi;
                u[0] = alo * blo - (_i - ahi * bhi - alo * bhi - ahi * blo);
                _j = cdxt_adyt1 * bdztail;
                c = splitter * cdxt_adyt1;
                ahi = c - (c - cdxt_adyt1);
                alo = cdxt_adyt1 - ahi;
                _0 = alo * blo - (_j - ahi * bhi - alo * bhi - ahi * blo);
                _k = _i + _0;
                bvirt = _k - _i;
                u[1] = _i - (_k - bvirt) + (_0 - bvirt);
                u3 = _j + _k;
                u[2] = _k - (u3 - _j);
                u[3] = u3;
                finlength = fast_expansion_sum_zeroelim(finlength, finnow, 4, u, finother);
                finswap = finnow; finnow = finother; finother = finswap;
            }
        }
        if (bdytail !== 0) {
            negate = -cdxtail;
            cdxt_bdyt1 = negate * bdytail;
            c = splitter * negate;
            ahi = c - (c - negate);
            alo = negate - ahi;
            c = splitter * bdytail;
            bhi = c - (c - bdytail);
            blo = bdytail - bhi;
            cdxt_bdyt0 = alo * blo - (cdxt_bdyt1 - ahi * bhi - alo * bhi - ahi * blo);
            c = splitter * adz;
            bhi = c - (c - adz);
            blo = adz - bhi;
            _i = cdxt_bdyt0 * adz;
            c = splitter * cdxt_bdyt0;
            ahi = c - (c - cdxt_bdyt0);
            alo = cdxt_bdyt0 - ahi;
            u[0] = alo * blo - (_i - ahi * bhi - alo * bhi - ahi * blo);
            _j = cdxt_bdyt1 * adz;
            c = splitter * cdxt_bdyt1;
            ahi = c - (c - cdxt_bdyt1);
            alo = cdxt_bdyt1 - ahi;
            _0 = alo * blo - (_j - ahi * bhi - alo * bhi - ahi * blo);
            _k = _i + _0;
            bvirt = _k - _i;
            u[1] = _i - (_k - bvirt) + (_0 - bvirt);
            u3 = _j + _k;
            u[2] = _k - (u3 - _j);
            u[3] = u3;
            finlength = fast_expansion_sum_zeroelim(finlength, finnow, 4, u, finother);
            finswap = finnow; finnow = finother; finother = finswap;
            if (adztail !== 0) {
                c = splitter * adztail;
                bhi = c - (c - adztail);
                blo = adztail - bhi;
                _i = cdxt_bdyt0 * adztail;
                c = splitter * cdxt_bdyt0;
                ahi = c - (c - cdxt_bdyt0);
                alo = cdxt_bdyt0 - ahi;
                u[0] = alo * blo - (_i - ahi * bhi - alo * bhi - ahi * blo);
                _j = cdxt_bdyt1 * adztail;
                c = splitter * cdxt_bdyt1;
                ahi = c - (c - cdxt_bdyt1);
                alo = cdxt_bdyt1 - ahi;
                _0 = alo * blo - (_j - ahi * bhi - alo * bhi - ahi * blo);
                _k = _i + _0;
                bvirt = _k - _i;
                u[1] = _i - (_k - bvirt) + (_0 - bvirt);
                u3 = _j + _k;
                u[2] = _k - (u3 - _j);
                u[3] = u3;
                finlength = fast_expansion_sum_zeroelim(finlength, finnow, 4, u, finother);
                finswap = finnow; finnow = finother; finother = finswap;
            }
        }
    }

    if (adztail !== 0) {
        wlength = scale_expansion_zeroelim(bctlen, bct, adztail, w);
        finlength = fast_expansion_sum_zeroelim(finlength, finnow, wlength, w, finother);
        finswap = finnow; finnow = finother; finother = finswap;
    }
    if (bdztail !== 0) {
        wlength = scale_expansion_zeroelim(catlen, cat, bdztail, w);
        finlength = fast_expansion_sum_zeroelim(finlength, finnow, wlength, w, finother);
        finswap = finnow; finnow = finother; finother = finswap;
    }
    if (cdztail !== 0) {
        wlength = scale_expansion_zeroelim(abtlen, abt, cdztail, w);
        finlength = fast_expansion_sum_zeroelim(finlength, finnow, wlength, w, finother);
        finswap = finnow; finnow = finother; finother = finswap;
    }

    return finnow[finlength - 1];
}

function orient3d(ax, ay, az, bx, by, bz, cx, cy, cz, dx, dy, dz) {
    const adx = ax - dx;
    const bdx = bx - dx;
    const cdx = cx - dx;
    const ady = ay - dy;
    const bdy = by - dy;
    const cdy = cy - dy;
    const adz = az - dz;
    const bdz = bz - dz;
    const cdz = cz - dz;

    const bdxcdy = bdx * cdy;
    const cdxbdy = cdx * bdy;

    const cdxady = cdx * ady;
    const adxcdy = adx * cdy;

    const adxbdy = adx * bdy;
    const bdxady = bdx * ady;

    const det =
        adz * (bdxcdy - cdxbdy) +
        bdz * (cdxady - adxcdy) +
        cdz * (adxbdy - bdxady);

    const permanent =
        (Math.abs(bdxcdy) + Math.abs(cdxbdy)) * Math.abs(adz) +
        (Math.abs(cdxady) + Math.abs(adxcdy)) * Math.abs(bdz) +
        (Math.abs(adxbdy) + Math.abs(bdxady)) * Math.abs(cdz);

    const errbound = o3derrboundA * permanent;
    if (det > errbound || -det > errbound) {
        return det;
    }

    return orient3dadapt(ax, ay, az, bx, by, bz, cx, cy, cz, dx, dy, dz, permanent);
}

function orient3dfast(ax, ay, az, bx, by, bz, cx, cy, cz, dx, dy, dz) {
    const adx = ax - dx;
    const bdx = bx - dx;
    const cdx = cx - dx;
    const ady = ay - dy;
    const bdy = by - dy;
    const cdy = cy - dy;
    const adz = az - dz;
    const bdz = bz - dz;
    const cdz = cz - dz;

    return adx * (bdy * cdz - bdz * cdy) +
        bdx * (cdy * adz - cdz * ady) +
        cdx * (ady * bdz - adz * bdy);
}

exports.orient3d = orient3d;
exports.orient3dfast = orient3dfast;

Object.defineProperty(exports, '__esModule', { value: true });

}));
