(function (global, factory) {
typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports) :
typeof define === 'function' && define.amd ? define(['exports'], factory) :
(global = global || self, factory(global.predicates = {}));
}(this, function (exports) { 'use strict';

const epsilon = 1.1102230246251565e-16;
const splitter = 134217729;
const resulterrbound = (3 + 8 * epsilon) * epsilon;

function fast_expansion_sum_zeroelim(elen, e, flen, f, h) {
    let Q, Qnew, hh, bvirt;
    let enow = e[0];
    let fnow = f[0];
    let eindex = 0;
    let findex = 0;
    if ((fnow > enow) === (fnow > -enow)) {
        Q = enow;
        enow = e[++eindex];
    } else {
        Q = fnow;
        fnow = f[++findex];
    }
    let hindex = 0;
    if ((eindex < elen) && (findex < flen)) {
        if ((fnow > enow) === (fnow > -enow)) {
            Qnew = enow + Q;
            hh = Q - (Qnew - enow);
            enow = e[++eindex];
        } else {
            Qnew = fnow + Q;
            hh = Q - (Qnew - fnow);
            fnow = f[++findex];
        }
        Q = Qnew;
        if (hh !== 0) {
            h[hindex++] = hh;
        }
        while ((eindex < elen) && (findex < flen)) {
            if ((fnow > enow) === (fnow > -enow)) {
                Qnew = Q + enow;
                bvirt = Qnew - Q;
                hh = Q - (Qnew - bvirt) + (enow - bvirt);
                enow = e[++eindex];
            } else {
                Qnew = Q + fnow;
                bvirt = Qnew - Q;
                hh = Q - (Qnew - bvirt) + (fnow - bvirt);
                fnow = f[++findex];
            }
            Q = Qnew;
            if (hh !== 0) {
                h[hindex++] = hh;
            }
        }
    }
    while (eindex < elen) {
        Qnew = Q + enow;
        bvirt = Qnew - Q;
        hh = Q - (Qnew - bvirt) + (enow - bvirt);
        enow = e[++eindex];
        Q = Qnew;
        if (hh !== 0) {
            h[hindex++] = hh;
        }
    }
    while (findex < flen) {
        Qnew = Q + fnow;
        bvirt = Qnew - Q;
        hh = Q - (Qnew - bvirt) + (fnow - bvirt);
        fnow = f[++findex];
        Q = Qnew;
        if (hh !== 0) {
            h[hindex++] = hh;
        }
    }
    if ((Q !== 0) || (hindex === 0)) {
        h[hindex++] = Q;
    }
    return hindex;
}

function scale_expansion_zeroelim(elen, e, b, h) {
    let Q, sum, hh, product1, product0;
    let bvirt, c, ahi, alo, bhi, blo;

    c = splitter * b;
    bhi = c - (c - b);
    blo = b - bhi;
    Q = e[0] * b;
    c = splitter * e[0];
    ahi = c - (c - e[0]);
    alo = e[0] - ahi;
    hh = alo * blo - (Q - ahi * bhi - alo * bhi - ahi * blo);
    let hindex = 0;
    if (hh !== 0) {
        h[hindex++] = hh;
    }
    for (let eindex = 1; eindex < elen; eindex++) {
        const enow = e[eindex];
        product1 = enow * b;
        c = splitter * enow;
        ahi = c - (c - enow);
        alo = enow - ahi;
        product0 = alo * blo - (product1 - ahi * bhi - alo * bhi - ahi * blo);
        sum = Q + product0;
        bvirt = sum - Q;
        hh = Q - (sum - bvirt) + (product0 - bvirt);
        if (hh !== 0) {
            h[hindex++] = hh;
        }
        Q = product1 + sum;
        hh = sum - (Q - product1);
        if (hh !== 0) {
            h[hindex++] = hh;
        }
    }
    if ((Q !== 0.0) || (hindex === 0)) {
        h[hindex++] = Q;
    }
    return hindex;
}

function estimate(elen, e) {
    let Q = e[0];
    for (let i = 1; i < elen; i++) Q += e[i];
    return Q;
}

function vec(n) {
    return new Float64Array(n);
}

const ccwerrboundA = (3 + 16 * epsilon) * epsilon;
const ccwerrboundB = (2 + 12 * epsilon) * epsilon;
const ccwerrboundC = (9 + 64 * epsilon) * epsilon * epsilon;

const B = vec(4);
const C1 = vec(8);
const C2 = vec(12);
const D = vec(16);
const u = vec(4);

function orient2dadapt(ax, ay, bx, by, cx, cy, detsum) {
    let acxtail, acytail, bcxtail, bcytail;
    let detleft, detright, detlefttail, detrighttail;
    let B3, u3, s1, t1, s0, t0;
    let bvirt, c, ahi, alo, bhi, blo, _i, _j, _0;

    const acx = ax - cx;
    const bcx = bx - cx;
    const acy = ay - cy;
    const bcy = by - cy;

    detleft = acx * bcy;
    c = splitter * acx;
    ahi = c - (c - acx);
    alo = acx - ahi;
    c = splitter * bcy;
    bhi = c - (c - bcy);
    blo = bcy - bhi;
    detlefttail = alo * blo - (detleft - ahi * bhi - alo * bhi - ahi * blo);
    detright = acy * bcx;
    c = splitter * acy;
    ahi = c - (c - acy);
    alo = acy - ahi;
    c = splitter * bcx;
    bhi = c - (c - bcx);
    blo = bcx - bhi;
    detrighttail = alo * blo - (detright - ahi * bhi - alo * bhi - ahi * blo);

    _i = detlefttail - detrighttail;
    bvirt = detlefttail - _i;
    B[0] = detlefttail - (_i + bvirt) + (bvirt - detrighttail);
    _j = detleft + _i;
    bvirt = _j - detleft;
    _0 = detleft - (_j - bvirt) + (_i - bvirt);
    _i = _0 - detright;
    bvirt = _0 - _i;
    B[1] = _0 - (_i + bvirt) + (bvirt - detright);
    B3 = _j + _i;
    bvirt = B3 - _j;
    B[2] = _j - (B3 - bvirt) + (_i - bvirt);
    B[3] = B3;

    let det = estimate(4, B);
    let errbound = ccwerrboundB * detsum;
    if (det >= errbound || -det >= errbound) return det;

    bvirt = ax - acx;
    acxtail = ax - (acx + bvirt) + (bvirt - cx);
    bvirt = bx - bcx;
    bcxtail = bx - (bcx + bvirt) + (bvirt - cx);
    bvirt = ay - acy;
    acytail = ay - (acy + bvirt) + (bvirt - cy);
    bvirt = by - bcy;
    bcytail = by - (bcy + bvirt) + (bvirt - cy);

    if (acxtail === 0 && acytail === 0 && bcxtail === 0 && bcytail === 0) return det;

    errbound = ccwerrboundC * detsum + resulterrbound * Math.abs(det);
    det += (acx * bcytail + bcy * acxtail) - (acy * bcxtail + bcx * acytail);
    if (det >= errbound || -det >= errbound) {
        return det;
    }

    s1 = acxtail * bcy;
    c = splitter * acxtail;
    ahi = c - (c - acxtail);
    alo = acxtail - ahi;
    c = splitter * bcy;
    bhi = c - (c - bcy);
    blo = bcy - bhi;
    s0 = alo * blo - (s1 - ahi * bhi - alo * bhi - ahi * blo);
    t1 = acytail * bcx;
    c = splitter * acytail;
    ahi = c - (c - acytail);
    alo = acytail - ahi;
    c = splitter * bcx;
    bhi = c - (c - bcx);
    blo = bcx - bhi;
    t0 = alo * blo - (t1 - ahi * bhi - alo * bhi - ahi * blo);
    _i = s0 - t0;
    bvirt = s0 - _i;
    u[0] = s0 - (_i + bvirt) + (bvirt - t0);
    _j = s1 + _i;
    bvirt = _j - s1;
    _0 = s1 - (_j - bvirt) + (_i - bvirt);
    _i = _0 - t1;
    bvirt = _0 - _i;
    u[1] = _0 - (_i + bvirt) + (bvirt - t1);
    u3 = _j + _i;
    bvirt = u3 - _j;
    u[2] = _j - (u3 - bvirt) + (_i - bvirt);
    u[3] = u3;
    const C1length = fast_expansion_sum_zeroelim(4, B, 4, u, C1);

    s1 = acx * bcytail;
    c = splitter * acx;
    ahi = c - (c - acx);
    alo = acx - ahi;
    c = splitter * bcytail;
    bhi = c - (c - bcytail);
    blo = bcytail - bhi;
    s0 = alo * blo - (s1 - ahi * bhi - alo * bhi - ahi * blo);
    t1 = acy * bcxtail;
    c = splitter * acy;
    ahi = c - (c - acy);
    alo = acy - ahi;
    c = splitter * bcxtail;
    bhi = c - (c - bcxtail);
    blo = bcxtail - bhi;
    t0 = alo * blo - (t1 - ahi * bhi - alo * bhi - ahi * blo);
    _i = s0 - t0;
    bvirt = s0 - _i;
    u[0] = s0 - (_i + bvirt) + (bvirt - t0);
    _j = s1 + _i;
    bvirt = _j - s1;
    _0 = s1 - (_j - bvirt) + (_i - bvirt);
    _i = _0 - t1;
    bvirt = _0 - _i;
    u[1] = _0 - (_i + bvirt) + (bvirt - t1);
    u3 = _j + _i;
    bvirt = u3 - _j;
    u[2] = _j - (u3 - bvirt) + (_i - bvirt);
    u[3] = u3;
    const C2length = fast_expansion_sum_zeroelim(C1length, C1, 4, u, C2);

    s1 = acxtail * bcytail;
    c = splitter * acxtail;
    ahi = c - (c - acxtail);
    alo = acxtail - ahi;
    c = splitter * bcytail;
    bhi = c - (c - bcytail);
    blo = bcytail - bhi;
    s0 = alo * blo - (s1 - ahi * bhi - alo * bhi - ahi * blo);
    t1 = acytail * bcxtail;
    c = splitter * acytail;
    ahi = c - (c - acytail);
    alo = acytail - ahi;
    c = splitter * bcxtail;
    bhi = c - (c - bcxtail);
    blo = bcxtail - bhi;
    t0 = alo * blo - (t1 - ahi * bhi - alo * bhi - ahi * blo);
    _i = s0 - t0;
    bvirt = s0 - _i;
    u[0] = s0 - (_i + bvirt) + (bvirt - t0);
    _j = s1 + _i;
    bvirt = _j - s1;
    _0 = s1 - (_j - bvirt) + (_i - bvirt);
    _i = _0 - t1;
    bvirt = _0 - _i;
    u[1] = _0 - (_i + bvirt) + (bvirt - t1);
    u3 = _j + _i;
    bvirt = u3 - _j;
    u[2] = _j - (u3 - bvirt) + (_i - bvirt);
    u[3] = u3;
    const Dlength = fast_expansion_sum_zeroelim(C2length, C2, 4, u, D);

    return D[Dlength - 1];
}

function orient2d(ax, ay, bx, by, cx, cy) {
    const detleft = (ay - cy) * (bx - cx);
    const detright = (ax - cx) * (by - cy);
    const det = detleft - detright;
    let detsum;

    if (detleft > 0) {
        if (detright <= 0) {
            return det;
        }
        detsum = detleft + detright;

    } else if (detleft < 0) {
        if (detright >= 0) {
            return det;
        }
        detsum = -detleft - detright;

    } else {
        return det;
    }

    const errbound = ccwerrboundA * detsum;
    if (det >= errbound || -det >= errbound) {
        return det;
    }

    return -orient2dadapt(ax, ay, bx, by, cx, cy, detsum);
}

function orient2dfast(ax, ay, bx, by, cx, cy) {
    return (ay - cy) * (bx - cx) - (ax - cx) * (by - cy);
}

const o3derrboundA = (7 + 56 * epsilon) * epsilon;
const o3derrboundB = (3 + 28 * epsilon) * epsilon;
const o3derrboundC = (26 + 288 * epsilon) * epsilon * epsilon;

const bc = vec(4);
const ca = vec(4);
const ab = vec(4);
const adet = vec(8);
const bdet = vec(8);
const cdet = vec(8);
const abdet = vec(16);
const fin1 = vec(192);
const fin2 = vec(192);
const at_b = vec(4);
const at_c = vec(4);
const bt_c = vec(4);
const bt_a = vec(4);
const ct_a = vec(4);
const ct_b = vec(4);
const bct = vec(8);
const cat = vec(8);
const abt = vec(8);
const u$1 = vec(4);
const v = vec(12);
const w = vec(16);

function orient3dadapt(ax, ay, az, bx, by, bz, cx, cy, cz, dx, dy, dz, permanent) {
    let bdxcdy1, cdxbdy1, cdxady1, adxcdy1, adxbdy1, bdxady1;
    let bdxcdy0, cdxbdy0, cdxady0, adxcdy0, adxbdy0, bdxady0;
    let bc3, ca3, ab3;
    let finnow, finother, finswap;
    let finlength;

    let adxtail, bdxtail, cdxtail;
    let adytail, bdytail, cdytail;
    let adztail, bdztail, cdztail;
    let at_blarge, at_clarge;
    let bt_clarge, bt_alarge;
    let ct_alarge, ct_blarge;
    let at_blen, at_clen, bt_clen, bt_alen, ct_alen, ct_blen;
    let bdxt_cdy1, cdxt_bdy1, cdxt_ady1;
    let adxt_cdy1, adxt_bdy1, bdxt_ady1;
    let bdxt_cdy0, cdxt_bdy0, cdxt_ady0;
    let adxt_cdy0, adxt_bdy0, bdxt_ady0;
    let bdyt_cdx1, cdyt_bdx1, cdyt_adx1;
    let adyt_cdx1, adyt_bdx1, bdyt_adx1;
    let bdyt_cdx0, cdyt_bdx0, cdyt_adx0;
    let adyt_cdx0, adyt_bdx0, bdyt_adx0;
    let bdxt_cdyt1, cdxt_bdyt1, cdxt_adyt1;
    let adxt_cdyt1, adxt_bdyt1, bdxt_adyt1;
    let bdxt_cdyt0, cdxt_bdyt0, cdxt_adyt0;
    let adxt_cdyt0, adxt_bdyt0, bdxt_adyt0;
    let u3;
    let vlength, wlength;
    let negate;

    let bvirt, c, ahi, alo, bhi, blo, _i, _j, _k, _0;

    const adx = ax - dx;
    const bdx = bx - dx;
    const cdx = cx - dx;
    const ady = ay - dy;
    const bdy = by - dy;
    const cdy = cy - dy;
    const adz = az - dz;
    const bdz = bz - dz;
    const cdz = cz - dz;

    bdxcdy1 = bdx * cdy;
    c = splitter * bdx;
    ahi = c - (c - bdx);
    alo = bdx - ahi;
    c = splitter * cdy;
    bhi = c - (c - cdy);
    blo = cdy - bhi;
    bdxcdy0 = alo * blo - (bdxcdy1 - ahi * bhi - alo * bhi - ahi * blo);
    cdxbdy1 = cdx * bdy;
    c = splitter * cdx;
    ahi = c - (c - cdx);
    alo = cdx - ahi;
    c = splitter * bdy;
    bhi = c - (c - bdy);
    blo = bdy - bhi;
    cdxbdy0 = alo * blo - (cdxbdy1 - ahi * bhi - alo * bhi - ahi * blo);
    _i = bdxcdy0 - cdxbdy0;
    bvirt = bdxcdy0 - _i;
    bc[0] = bdxcdy0 - (_i + bvirt) + (bvirt - cdxbdy0);
    _j = bdxcdy1 + _i;
    bvirt = _j - bdxcdy1;
    _0 = bdxcdy1 - (_j - bvirt) + (_i - bvirt);
    _i = _0 - cdxbdy1;
    bvirt = _0 - _i;
    bc[1] = _0 - (_i + bvirt) + (bvirt - cdxbdy1);
    bc3 = _j + _i;
    bvirt = bc3 - _j;
    bc[2] = _j - (bc3 - bvirt) + (_i - bvirt);
    bc[3] = bc3;
    const alen = scale_expansion_zeroelim(4, bc, adz, adet);

    cdxady1 = cdx * ady;
    c = splitter * cdx;
    ahi = c - (c - cdx);
    alo = cdx - ahi;
    c = splitter * ady;
    bhi = c - (c - ady);
    blo = ady - bhi;
    cdxady0 = alo * blo - (cdxady1 - ahi * bhi - alo * bhi - ahi * blo);
    adxcdy1 = adx * cdy;
    c = splitter * adx;
    ahi = c - (c - adx);
    alo = adx - ahi;
    c = splitter * cdy;
    bhi = c - (c - cdy);
    blo = cdy - bhi;
    adxcdy0 = alo * blo - (adxcdy1 - ahi * bhi - alo * bhi - ahi * blo);
    _i = cdxady0 - adxcdy0;
    bvirt = cdxady0 - _i;
    ca[0] = cdxady0 - (_i + bvirt) + (bvirt - adxcdy0);
    _j = cdxady1 + _i;
    bvirt = _j - cdxady1;
    _0 = cdxady1 - (_j - bvirt) + (_i - bvirt);
    _i = _0 - adxcdy1;
    bvirt = _0 - _i;
    ca[1] = _0 - (_i + bvirt) + (bvirt - adxcdy1);
    ca3 = _j + _i;
    bvirt = ca3 - _j;
    ca[2] = _j - (ca3 - bvirt) + (_i - bvirt);
    ca[3] = ca3;
    const blen = scale_expansion_zeroelim(4, ca, bdz, bdet);

    adxbdy1 = adx * bdy;
    c = splitter * adx;
    ahi = c - (c - adx);
    alo = adx - ahi;
    c = splitter * bdy;
    bhi = c - (c - bdy);
    blo = bdy - bhi;
    adxbdy0 = alo * blo - (adxbdy1 - ahi * bhi - alo * bhi - ahi * blo);
    bdxady1 = bdx * ady;
    c = splitter * bdx;
    ahi = c - (c - bdx);
    alo = bdx - ahi;
    c = splitter * ady;
    bhi = c - (c - ady);
    blo = ady - bhi;
    bdxady0 = alo * blo - (bdxady1 - ahi * bhi - alo * bhi - ahi * blo);
    _i = adxbdy0 - bdxady0;
    bvirt = adxbdy0 - _i;
    ab[0] = adxbdy0 - (_i + bvirt) + (bvirt - bdxady0);
    _j = adxbdy1 + _i;
    bvirt = _j - adxbdy1;
    _0 = adxbdy1 - (_j - bvirt) + (_i - bvirt);
    _i = _0 - bdxady1;
    bvirt = _0 - _i;
    ab[1] = _0 - (_i + bvirt) + (bvirt - bdxady1);
    ab3 = _j + _i;
    bvirt = ab3 - _j;
    ab[2] = _j - (ab3 - bvirt) + (_i - bvirt);
    ab[3] = ab3;
    const clen = scale_expansion_zeroelim(4, ab, cdz, cdet);

    const ablen = fast_expansion_sum_zeroelim(alen, adet, blen, bdet, abdet);
    finlength = fast_expansion_sum_zeroelim(ablen, abdet, clen, cdet, fin1);

    let det = estimate(finlength, fin1);
    let errbound = o3derrboundB * permanent;
    if (det >= errbound || -det >= errbound) {
        return det;
    }

    bvirt = ax - adx;
    adxtail = ax - (adx + bvirt) + (bvirt - dx);
    bvirt = bx - bdx;
    bdxtail = bx - (bdx + bvirt) + (bvirt - dx);
    bvirt = cx - cdx;
    cdxtail = cx - (cdx + bvirt) + (bvirt - dx);
    bvirt = ay - ady;
    adytail = ay - (ady + bvirt) + (bvirt - dy);
    bvirt = by - bdy;
    bdytail = by - (bdy + bvirt) + (bvirt - dy);
    bvirt = cy - cdy;
    cdytail = cy - (cdy + bvirt) + (bvirt - dy);
    bvirt = az - adz;
    adztail = az - (adz + bvirt) + (bvirt - dz);
    bvirt = bz - bdz;
    bdztail = bz - (bdz + bvirt) + (bvirt - dz);
    bvirt = cz - cdz;
    cdztail = cz - (cdz + bvirt) + (bvirt - dz);

    if (adxtail === 0 && bdxtail === 0 && cdxtail === 0 &&
        adytail === 0 && bdytail === 0 && cdytail === 0 &&
        adztail === 0 && bdztail === 0 && cdztail === 0) {
        return det;
    }

    errbound = o3derrboundC * permanent + resulterrbound * Math.abs(det);
    det +=
        adz * (bdx * cdytail + cdy * bdxtail - (bdy * cdxtail + cdx * bdytail)) + adztail * (bdx * cdy - bdy * cdx) +
        bdz * (cdx * adytail + ady * cdxtail - (cdy * adxtail + adx * cdytail)) + bdztail * (cdx * ady - cdy * adx) +
        cdz * (adx * bdytail + bdy * adxtail - (ady * bdxtail + bdx * adytail)) + cdztail * (adx * bdy - ady * bdx);
    if (det >= errbound || -det >= errbound) {
        return det;
    }

    finnow = fin1;
    finother = fin2;

    if (adxtail === 0) {
        if (adytail === 0) {
            at_b[0] = 0;
            at_blen = 1;
            at_c[0] = 0;
            at_clen = 1;
        } else {
            negate = -adytail;
            at_blarge = negate * bdx;
            c = splitter * negate;
            ahi = c - (c - negate);
            alo = negate - ahi;
            c = splitter * bdx;
            bhi = c - (c - bdx);
            blo = bdx - bhi;
            at_b[0] = alo * blo - (at_blarge - ahi * bhi - alo * bhi - ahi * blo);
            at_b[1] = at_blarge;
            at_blen = 2;
            at_clarge = adytail * cdx;
            c = splitter * adytail;
            ahi = c - (c - adytail);
            alo = adytail - ahi;
            c = splitter * cdx;
            bhi = c - (c - cdx);
            blo = cdx - bhi;
            at_c[0] = alo * blo - (at_clarge - ahi * bhi - alo * bhi - ahi * blo);
            at_c[1] = at_clarge;
            at_clen = 2;
        }
    } else {
        if (adytail === 0) {
            at_blarge = adxtail * bdy;
            c = splitter * adxtail;
            ahi = c - (c - adxtail);
            alo = adxtail - ahi;
            c = splitter * bdy;
            bhi = c - (c - bdy);
            blo = bdy - bhi;
            at_b[0] = alo * blo - (at_blarge - ahi * bhi - alo * bhi - ahi * blo);
            at_b[1] = at_blarge;
            at_blen = 2;
            negate = -adxtail;
            at_clarge = negate * cdy;
            c = splitter * negate;
            ahi = c - (c - negate);
            alo = negate - ahi;
            c = splitter * cdy;
            bhi = c - (c - cdy);
            blo = cdy - bhi;
            at_c[0] = alo * blo - (at_clarge - ahi * bhi - alo * bhi - ahi * blo);
            at_c[1] = at_clarge;
            at_clen = 2;
        } else {
            adxt_bdy1 = adxtail * bdy;
            c = splitter * adxtail;
            ahi = c - (c - adxtail);
            alo = adxtail - ahi;
            c = splitter * bdy;
            bhi = c - (c - bdy);
            blo = bdy - bhi;
            adxt_bdy0 = alo * blo - (adxt_bdy1 - ahi * bhi - alo * bhi - ahi * blo);
            adyt_bdx1 = adytail * bdx;
            c = splitter * adytail;
            ahi = c - (c - adytail);
            alo = adytail - ahi;
            c = splitter * bdx;
            bhi = c - (c - bdx);
            blo = bdx - bhi;
            adyt_bdx0 = alo * blo - (adyt_bdx1 - ahi * bhi - alo * bhi - ahi * blo);
            _i = adxt_bdy0 - adyt_bdx0;
            bvirt = adxt_bdy0 - _i;
            at_b[0] = adxt_bdy0 - (_i + bvirt) + (bvirt - adyt_bdx0);
            _j = adxt_bdy1 + _i;
            bvirt = _j - adxt_bdy1;
            _0 = adxt_bdy1 - (_j - bvirt) + (_i - bvirt);
            _i = _0 - adyt_bdx1;
            bvirt = _0 - _i;
            at_b[1] = _0 - (_i + bvirt) + (bvirt - adyt_bdx1);
            at_blarge = _j + _i;
            bvirt = at_blarge - _j;
            at_b[2] = _j - (at_blarge - bvirt) + (_i - bvirt);
            at_b[3] = at_blarge;
            at_blen = 4;
            adyt_cdx1 = adytail * cdx;
            c = splitter * adytail;
            ahi = c - (c - adytail);
            alo = adytail - ahi;
            c = splitter * cdx;
            bhi = c - (c - cdx);
            blo = cdx - bhi;
            adyt_cdx0 = alo * blo - (adyt_cdx1 - ahi * bhi - alo * bhi - ahi * blo);
            adxt_cdy1 = adxtail * cdy;
            c = splitter * adxtail;
            ahi = c - (c - adxtail);
            alo = adxtail - ahi;
            c = splitter * cdy;
            bhi = c - (c - cdy);
            blo = cdy - bhi;
            adxt_cdy0 = alo * blo - (adxt_cdy1 - ahi * bhi - alo * bhi - ahi * blo);
            _i = adyt_cdx0 - adxt_cdy0;
            bvirt = adyt_cdx0 - _i;
            at_c[0] = adyt_cdx0 - (_i + bvirt) + (bvirt - adxt_cdy0);
            _j = adyt_cdx1 + _i;
            bvirt = _j - adyt_cdx1;
            _0 = adyt_cdx1 - (_j - bvirt) + (_i - bvirt);
            _i = _0 - adxt_cdy1;
            bvirt = _0 - _i;
            at_c[1] = _0 - (_i + bvirt) + (bvirt - adxt_cdy1);
            at_clarge = _j + _i;
            bvirt = at_clarge - _j;
            at_c[2] = _j - (at_clarge - bvirt) + (_i - bvirt);
            at_c[3] = at_clarge;
            at_clen = 4;
        }
    }
    if (bdxtail === 0) {
        if (bdytail === 0) {
            bt_c[0] = 0;
            bt_clen = 1;
            bt_a[0] = 0;
            bt_alen = 1;
        } else {
            negate = -bdytail;
            bt_clarge = negate * cdx;
            c = splitter * negate;
            ahi = c - (c - negate);
            alo = negate - ahi;
            c = splitter * cdx;
            bhi = c - (c - cdx);
            blo = cdx - bhi;
            bt_c[0] = alo * blo - (bt_clarge - ahi * bhi - alo * bhi - ahi * blo);
            bt_c[1] = bt_clarge;
            bt_clen = 2;
            bt_alarge = bdytail * adx;
            c = splitter * bdytail;
            ahi = c - (c - bdytail);
            alo = bdytail - ahi;
            c = splitter * adx;
            bhi = c - (c - adx);
            blo = adx - bhi;
            bt_a[0] = alo * blo - (bt_alarge - ahi * bhi - alo * bhi - ahi * blo);
            bt_a[1] = bt_alarge;
            bt_alen = 2;
        }
    } else {
        if (bdytail === 0) {
            bt_clarge = bdxtail * cdy;
            c = splitter * bdxtail;
            ahi = c - (c - bdxtail);
            alo = bdxtail - ahi;
            c = splitter * cdy;
            bhi = c - (c - cdy);
            blo = cdy - bhi;
            bt_c[0] = alo * blo - (bt_clarge - ahi * bhi - alo * bhi - ahi * blo);
            bt_c[1] = bt_clarge;
            bt_clen = 2;
            negate = -bdxtail;
            bt_alarge = negate * ady;
            c = splitter * negate;
            ahi = c - (c - negate);
            alo = negate - ahi;
            c = splitter * ady;
            bhi = c - (c - ady);
            blo = ady - bhi;
            bt_a[0] = alo * blo - (bt_alarge - ahi * bhi - alo * bhi - ahi * blo);
            bt_a[1] = bt_alarge;
            bt_alen = 2;
        } else {
            bdxt_cdy1 = bdxtail * cdy;
            c = splitter * bdxtail;
            ahi = c - (c - bdxtail);
            alo = bdxtail - ahi;
            c = splitter * cdy;
            bhi = c - (c - cdy);
            blo = cdy - bhi;
            bdxt_cdy0 = alo * blo - (bdxt_cdy1 - ahi * bhi - alo * bhi - ahi * blo);
            bdyt_cdx1 = bdytail * cdx;
            c = splitter * bdytail;
            ahi = c - (c - bdytail);
            alo = bdytail - ahi;
            c = splitter * cdx;
            bhi = c - (c - cdx);
            blo = cdx - bhi;
            bdyt_cdx0 = alo * blo - (bdyt_cdx1 - ahi * bhi - alo * bhi - ahi * blo);
            _i = bdxt_cdy0 - bdyt_cdx0;
            bvirt = bdxt_cdy0 - _i;
            bt_c[0] = bdxt_cdy0 - (_i + bvirt) + (bvirt - bdyt_cdx0);
            _j = bdxt_cdy1 + _i;
            bvirt = _j - bdxt_cdy1;
            _0 = bdxt_cdy1 - (_j - bvirt) + (_i - bvirt);
            _i = _0 - bdyt_cdx1;
            bvirt = _0 - _i;
            bt_c[1] = _0 - (_i + bvirt) + (bvirt - bdyt_cdx1);
            bt_clarge = _j + _i;
            bvirt = bt_clarge - _j;
            bt_c[2] = _j - (bt_clarge - bvirt) + (_i - bvirt);
            bt_c[3] = bt_clarge;
            bt_clen = 4;
            bdyt_adx1 = bdytail * adx;
            c = splitter * bdytail;
            ahi = c - (c - bdytail);
            alo = bdytail - ahi;
            c = splitter * adx;
            bhi = c - (c - adx);
            blo = adx - bhi;
            bdyt_adx0 = alo * blo - (bdyt_adx1 - ahi * bhi - alo * bhi - ahi * blo);
            bdxt_ady1 = bdxtail * ady;
            c = splitter * bdxtail;
            ahi = c - (c - bdxtail);
            alo = bdxtail - ahi;
            c = splitter * ady;
            bhi = c - (c - ady);
            blo = ady - bhi;
            bdxt_ady0 = alo * blo - (bdxt_ady1 - ahi * bhi - alo * bhi - ahi * blo);
            _i = bdyt_adx0 - bdxt_ady0;
            bvirt = bdyt_adx0 - _i;
            bt_a[0] = bdyt_adx0 - (_i + bvirt) + (bvirt - bdxt_ady0);
            _j = bdyt_adx1 + _i;
            bvirt = _j - bdyt_adx1;
            _0 = bdyt_adx1 - (_j - bvirt) + (_i - bvirt);
            _i = _0 - bdxt_ady1;
            bvirt = _0 - _i;
            bt_a[1] = _0 - (_i + bvirt) + (bvirt - bdxt_ady1);
            bt_alarge = _j + _i;
            bvirt = bt_alarge - _j;
            bt_a[2] = _j - (bt_alarge - bvirt) + (_i - bvirt);
            bt_a[3] = bt_alarge;
            bt_alen = 4;
        }
    }
    if (cdxtail === 0) {
        if (cdytail === 0) {
            ct_a[0] = 0;
            ct_alen = 1;
            ct_b[0] = 0;
            ct_blen = 1;
        } else {
            negate = -cdytail;
            ct_alarge = negate * adx;
            c = splitter * negate;
            ahi = c - (c - negate);
            alo = negate - ahi;
            c = splitter * adx;
            bhi = c - (c - adx);
            blo = adx - bhi;
            ct_a[0] = alo * blo - (ct_alarge - ahi * bhi - alo * bhi - ahi * blo);
            ct_a[1] = ct_alarge;
            ct_alen = 2;
            ct_blarge = cdytail * bdx;
            c = splitter * cdytail;
            ahi = c - (c - cdytail);
            alo = cdytail - ahi;
            c = splitter * bdx;
            bhi = c - (c - bdx);
            blo = bdx - bhi;
            ct_b[0] = alo * blo - (ct_blarge - ahi * bhi - alo * bhi - ahi * blo);
            ct_b[1] = ct_blarge;
            ct_blen = 2;
        }
    } else {
        if (cdytail === 0) {
            ct_alarge = cdxtail * ady;
            c = splitter * cdxtail;
            ahi = c - (c - cdxtail);
            alo = cdxtail - ahi;
            c = splitter * ady;
            bhi = c - (c - ady);
            blo = ady - bhi;
            ct_a[0] = alo * blo - (ct_alarge - ahi * bhi - alo * bhi - ahi * blo);
            ct_a[1] = ct_alarge;
            ct_alen = 2;
            negate = -cdxtail;
            ct_blarge = negate * bdy;
            c = splitter * negate;
            ahi = c - (c - negate);
            alo = negate - ahi;
            c = splitter * bdy;
            bhi = c - (c - bdy);
            blo = bdy - bhi;
            ct_b[0] = alo * blo - (ct_blarge - ahi * bhi - alo * bhi - ahi * blo);
            ct_b[1] = ct_blarge;
            ct_blen = 2;
        } else {
            cdxt_ady1 = cdxtail * ady;
            c = splitter * cdxtail;
            ahi = c - (c - cdxtail);
            alo = cdxtail - ahi;
            c = splitter * ady;
            bhi = c - (c - ady);
            blo = ady - bhi;
            cdxt_ady0 = alo * blo - (cdxt_ady1 - ahi * bhi - alo * bhi - ahi * blo);
            cdyt_adx1 = cdytail * adx;
            c = splitter * cdytail;
            ahi = c - (c - cdytail);
            alo = cdytail - ahi;
            c = splitter * adx;
            bhi = c - (c - adx);
            blo = adx - bhi;
            cdyt_adx0 = alo * blo - (cdyt_adx1 - ahi * bhi - alo * bhi - ahi * blo);
            _i = cdxt_ady0 - cdyt_adx0;
            bvirt = cdxt_ady0 - _i;
            ct_a[0] = cdxt_ady0 - (_i + bvirt) + (bvirt - cdyt_adx0);
            _j = cdxt_ady1 + _i;
            bvirt = _j - cdxt_ady1;
            _0 = cdxt_ady1 - (_j - bvirt) + (_i - bvirt);
            _i = _0 - cdyt_adx1;
            bvirt = _0 - _i;
            ct_a[1] = _0 - (_i + bvirt) + (bvirt - cdyt_adx1);
            ct_alarge = _j + _i;
            bvirt = ct_alarge - _j;
            ct_a[2] = _j - (ct_alarge - bvirt) + (_i - bvirt);
            ct_a[3] = ct_alarge;
            ct_alen = 4;
            cdyt_bdx1 = cdytail * bdx;
            c = splitter * cdytail;
            ahi = c - (c - cdytail);
            alo = cdytail - ahi;
            c = splitter * bdx;
            bhi = c - (c - bdx);
            blo = bdx - bhi;
            cdyt_bdx0 = alo * blo - (cdyt_bdx1 - ahi * bhi - alo * bhi - ahi * blo);
            cdxt_bdy1 = cdxtail * bdy;
            c = splitter * cdxtail;
            ahi = c - (c - cdxtail);
            alo = cdxtail - ahi;
            c = splitter * bdy;
            bhi = c - (c - bdy);
            blo = bdy - bhi;
            cdxt_bdy0 = alo * blo - (cdxt_bdy1 - ahi * bhi - alo * bhi - ahi * blo);
            _i = cdyt_bdx0 - cdxt_bdy0;
            bvirt = cdyt_bdx0 - _i;
            ct_b[0] = cdyt_bdx0 - (_i + bvirt) + (bvirt - cdxt_bdy0);
            _j = cdyt_bdx1 + _i;
            bvirt = _j - cdyt_bdx1;
            _0 = cdyt_bdx1 - (_j - bvirt) + (_i - bvirt);
            _i = _0 - cdxt_bdy1;
            bvirt = _0 - _i;
            ct_b[1] = _0 - (_i + bvirt) + (bvirt - cdxt_bdy1);
            ct_blarge = _j + _i;
            bvirt = ct_blarge - _j;
            ct_b[2] = _j - (ct_blarge - bvirt) + (_i - bvirt);
            ct_b[3] = ct_blarge;
            ct_blen = 4;
        }
    }

    const bctlen = fast_expansion_sum_zeroelim(bt_clen, bt_c, ct_blen, ct_b, bct);
    wlength = scale_expansion_zeroelim(bctlen, bct, adz, w);
    finlength = fast_expansion_sum_zeroelim(finlength, finnow, wlength, w, finother);
    finswap = finnow; finnow = finother; finother = finswap;

    const catlen = fast_expansion_sum_zeroelim(ct_alen, ct_a, at_clen, at_c, cat);
    wlength = scale_expansion_zeroelim(catlen, cat, bdz, w);
    finlength = fast_expansion_sum_zeroelim(finlength, finnow, wlength, w, finother);
    finswap = finnow; finnow = finother; finother = finswap;

    const abtlen = fast_expansion_sum_zeroelim(at_blen, at_b, bt_alen, bt_a, abt);
    wlength = scale_expansion_zeroelim(abtlen, abt, cdz, w);
    finlength = fast_expansion_sum_zeroelim(finlength, finnow, wlength, w, finother);
    finswap = finnow; finnow = finother; finother = finswap;

    if (adztail !== 0) {
        vlength = scale_expansion_zeroelim(4, bc, adztail, v);
        finlength = fast_expansion_sum_zeroelim(finlength, finnow, vlength, v, finother);
        finswap = finnow; finnow = finother; finother = finswap;
    }
    if (bdztail !== 0) {
        vlength = scale_expansion_zeroelim(4, ca, bdztail, v);
        finlength = fast_expansion_sum_zeroelim(finlength, finnow, vlength, v, finother);
        finswap = finnow; finnow = finother; finother = finswap;
    }
    if (cdztail !== 0) {
        vlength = scale_expansion_zeroelim(4, ab, cdztail, v);
        finlength = fast_expansion_sum_zeroelim(finlength, finnow, vlength, v, finother);
        finswap = finnow; finnow = finother; finother = finswap;
    }

    if (adxtail !== 0) {
        if (bdytail !== 0) {
            adxt_bdyt1 = adxtail * bdytail;
            c = splitter * adxtail;
            ahi = c - (c - adxtail);
            alo = adxtail - ahi;
            c = splitter * bdytail;
            bhi = c - (c - bdytail);
            blo = bdytail - bhi;
            adxt_bdyt0 = alo * blo - (adxt_bdyt1 - ahi * bhi - alo * bhi - ahi * blo);
            c = splitter * cdz;
            bhi = c - (c - cdz);
            blo = cdz - bhi;
            _i = adxt_bdyt0 * cdz;
            c = splitter * adxt_bdyt0;
            ahi = c - (c - adxt_bdyt0);
            alo = adxt_bdyt0 - ahi;
            u$1[0] = alo * blo - (_i - ahi * bhi - alo * bhi - ahi * blo);
            _j = adxt_bdyt1 * cdz;
            c = splitter * adxt_bdyt1;
            ahi = c - (c - adxt_bdyt1);
            alo = adxt_bdyt1 - ahi;
            _0 = alo * blo - (_j - ahi * bhi - alo * bhi - ahi * blo);
            _k = _i + _0;
            bvirt = _k - _i;
            u$1[1] = _i - (_k - bvirt) + (_0 - bvirt);
            u3 = _j + _k;
            u$1[2] = _k - (u3 - _j);
            u$1[3] = u3;
            finlength = fast_expansion_sum_zeroelim(finlength, finnow, 4, u$1, finother);
            finswap = finnow; finnow = finother; finother = finswap;
            if (cdztail !== 0) {
                c = splitter * cdztail;
                bhi = c - (c - cdztail);
                blo = cdztail - bhi;
                _i = adxt_bdyt0 * cdztail;
                c = splitter * adxt_bdyt0;
                ahi = c - (c - adxt_bdyt0);
                alo = adxt_bdyt0 - ahi;
                u$1[0] = alo * blo - (_i - ahi * bhi - alo * bhi - ahi * blo);
                _j = adxt_bdyt1 * cdztail;
                c = splitter * adxt_bdyt1;
                ahi = c - (c - adxt_bdyt1);
                alo = adxt_bdyt1 - ahi;
                _0 = alo * blo - (_j - ahi * bhi - alo * bhi - ahi * blo);
                _k = _i + _0;
                bvirt = _k - _i;
                u$1[1] = _i - (_k - bvirt) + (_0 - bvirt);
                u3 = _j + _k;
                u$1[2] = _k - (u3 - _j);
                u$1[3] = u3;
                finlength = fast_expansion_sum_zeroelim(finlength, finnow, 4, u$1, finother);
                finswap = finnow; finnow = finother; finother = finswap;
            }
        }
        if (cdytail !== 0) {
            negate = -adxtail;
            adxt_cdyt1 = negate * cdytail;
            c = splitter * negate;
            ahi = c - (c - negate);
            alo = negate - ahi;
            c = splitter * cdytail;
            bhi = c - (c - cdytail);
            blo = cdytail - bhi;
            adxt_cdyt0 = alo * blo - (adxt_cdyt1 - ahi * bhi - alo * bhi - ahi * blo);
            c = splitter * bdz;
            bhi = c - (c - bdz);
            blo = bdz - bhi;
            _i = adxt_cdyt0 * bdz;
            c = splitter * adxt_cdyt0;
            ahi = c - (c - adxt_cdyt0);
            alo = adxt_cdyt0 - ahi;
            u$1[0] = alo * blo - (_i - ahi * bhi - alo * bhi - ahi * blo);
            _j = adxt_cdyt1 * bdz;
            c = splitter * adxt_cdyt1;
            ahi = c - (c - adxt_cdyt1);
            alo = adxt_cdyt1 - ahi;
            _0 = alo * blo - (_j - ahi * bhi - alo * bhi - ahi * blo);
            _k = _i + _0;
            bvirt = _k - _i;
            u$1[1] = _i - (_k - bvirt) + (_0 - bvirt);
            u3 = _j + _k;
            u$1[2] = _k - (u3 - _j);
            u$1[3] = u3;
            finlength = fast_expansion_sum_zeroelim(finlength, finnow, 4, u$1, finother);
            finswap = finnow; finnow = finother; finother = finswap;
            if (bdztail !== 0) {
                c = splitter * bdztail;
                bhi = c - (c - bdztail);
                blo = bdztail - bhi;
                _i = adxt_cdyt0 * bdztail;
                c = splitter * adxt_cdyt0;
                ahi = c - (c - adxt_cdyt0);
                alo = adxt_cdyt0 - ahi;
                u$1[0] = alo * blo - (_i - ahi * bhi - alo * bhi - ahi * blo);
                _j = adxt_cdyt1 * bdztail;
                c = splitter * adxt_cdyt1;
                ahi = c - (c - adxt_cdyt1);
                alo = adxt_cdyt1 - ahi;
                _0 = alo * blo - (_j - ahi * bhi - alo * bhi - ahi * blo);
                _k = _i + _0;
                bvirt = _k - _i;
                u$1[1] = _i - (_k - bvirt) + (_0 - bvirt);
                u3 = _j + _k;
                u$1[2] = _k - (u3 - _j);
                u$1[3] = u3;
                finlength = fast_expansion_sum_zeroelim(finlength, finnow, 4, u$1, finother);
                finswap = finnow; finnow = finother; finother = finswap;
            }
        }
    }
    if (bdxtail !== 0) {
        if (cdytail !== 0) {
            bdxt_cdyt1 = bdxtail * cdytail;
            c = splitter * bdxtail;
            ahi = c - (c - bdxtail);
            alo = bdxtail - ahi;
            c = splitter * cdytail;
            bhi = c - (c - cdytail);
            blo = cdytail - bhi;
            bdxt_cdyt0 = alo * blo - (bdxt_cdyt1 - ahi * bhi - alo * bhi - ahi * blo);
            c = splitter * adz;
            bhi = c - (c - adz);
            blo = adz - bhi;
            _i = bdxt_cdyt0 * adz;
            c = splitter * bdxt_cdyt0;
            ahi = c - (c - bdxt_cdyt0);
            alo = bdxt_cdyt0 - ahi;
            u$1[0] = alo * blo - (_i - ahi * bhi - alo * bhi - ahi * blo);
            _j = bdxt_cdyt1 * adz;
            c = splitter * bdxt_cdyt1;
            ahi = c - (c - bdxt_cdyt1);
            alo = bdxt_cdyt1 - ahi;
            _0 = alo * blo - (_j - ahi * bhi - alo * bhi - ahi * blo);
            _k = _i + _0;
            bvirt = _k - _i;
            u$1[1] = _i - (_k - bvirt) + (_0 - bvirt);
            u3 = _j + _k;
            u$1[2] = _k - (u3 - _j);
            u$1[3] = u3;
            finlength = fast_expansion_sum_zeroelim(finlength, finnow, 4, u$1, finother);
            finswap = finnow; finnow = finother; finother = finswap;
            if (adztail !== 0) {
                c = splitter * adztail;
                bhi = c - (c - adztail);
                blo = adztail - bhi;
                _i = bdxt_cdyt0 * adztail;
                c = splitter * bdxt_cdyt0;
                ahi = c - (c - bdxt_cdyt0);
                alo = bdxt_cdyt0 - ahi;
                u$1[0] = alo * blo - (_i - ahi * bhi - alo * bhi - ahi * blo);
                _j = bdxt_cdyt1 * adztail;
                c = splitter * bdxt_cdyt1;
                ahi = c - (c - bdxt_cdyt1);
                alo = bdxt_cdyt1 - ahi;
                _0 = alo * blo - (_j - ahi * bhi - alo * bhi - ahi * blo);
                _k = _i + _0;
                bvirt = _k - _i;
                u$1[1] = _i - (_k - bvirt) + (_0 - bvirt);
                u3 = _j + _k;
                u$1[2] = _k - (u3 - _j);
                u$1[3] = u3;
                finlength = fast_expansion_sum_zeroelim(finlength, finnow, 4, u$1, finother);
                finswap = finnow; finnow = finother; finother = finswap;
            }
        }
        if (adytail !== 0) {
            negate = -bdxtail;
            bdxt_adyt1 = negate * adytail;
            c = splitter * negate;
            ahi = c - (c - negate);
            alo = negate - ahi;
            c = splitter * adytail;
            bhi = c - (c - adytail);
            blo = adytail - bhi;
            bdxt_adyt0 = alo * blo - (bdxt_adyt1 - ahi * bhi - alo * bhi - ahi * blo);
            c = splitter * cdz;
            bhi = c - (c - cdz);
            blo = cdz - bhi;
            _i = bdxt_adyt0 * cdz;
            c = splitter * bdxt_adyt0;
            ahi = c - (c - bdxt_adyt0);
            alo = bdxt_adyt0 - ahi;
            u$1[0] = alo * blo - (_i - ahi * bhi - alo * bhi - ahi * blo);
            _j = bdxt_adyt1 * cdz;
            c = splitter * bdxt_adyt1;
            ahi = c - (c - bdxt_adyt1);
            alo = bdxt_adyt1 - ahi;
            _0 = alo * blo - (_j - ahi * bhi - alo * bhi - ahi * blo);
            _k = _i + _0;
            bvirt = _k - _i;
            u$1[1] = _i - (_k - bvirt) + (_0 - bvirt);
            u3 = _j + _k;
            u$1[2] = _k - (u3 - _j);
            u$1[3] = u3;
            finlength = fast_expansion_sum_zeroelim(finlength, finnow, 4, u$1, finother);
            finswap = finnow; finnow = finother; finother = finswap;
            if (cdztail !== 0) {
                c = splitter * cdztail;
                bhi = c - (c - cdztail);
                blo = cdztail - bhi;
                _i = bdxt_adyt0 * cdztail;
                c = splitter * bdxt_adyt0;
                ahi = c - (c - bdxt_adyt0);
                alo = bdxt_adyt0 - ahi;
                u$1[0] = alo * blo - (_i - ahi * bhi - alo * bhi - ahi * blo);
                _j = bdxt_adyt1 * cdztail;
                c = splitter * bdxt_adyt1;
                ahi = c - (c - bdxt_adyt1);
                alo = bdxt_adyt1 - ahi;
                _0 = alo * blo - (_j - ahi * bhi - alo * bhi - ahi * blo);
                _k = _i + _0;
                bvirt = _k - _i;
                u$1[1] = _i - (_k - bvirt) + (_0 - bvirt);
                u3 = _j + _k;
                u$1[2] = _k - (u3 - _j);
                u$1[3] = u3;
                finlength = fast_expansion_sum_zeroelim(finlength, finnow, 4, u$1, finother);
                finswap = finnow; finnow = finother; finother = finswap;
            }
        }
    }
    if (cdxtail !== 0) {
        if (adytail !== 0) {
            cdxt_adyt1 = cdxtail * adytail;
            c = splitter * cdxtail;
            ahi = c - (c - cdxtail);
            alo = cdxtail - ahi;
            c = splitter * adytail;
            bhi = c - (c - adytail);
            blo = adytail - bhi;
            cdxt_adyt0 = alo * blo - (cdxt_adyt1 - ahi * bhi - alo * bhi - ahi * blo);
            c = splitter * bdz;
            bhi = c - (c - bdz);
            blo = bdz - bhi;
            _i = cdxt_adyt0 * bdz;
            c = splitter * cdxt_adyt0;
            ahi = c - (c - cdxt_adyt0);
            alo = cdxt_adyt0 - ahi;
            u$1[0] = alo * blo - (_i - ahi * bhi - alo * bhi - ahi * blo);
            _j = cdxt_adyt1 * bdz;
            c = splitter * cdxt_adyt1;
            ahi = c - (c - cdxt_adyt1);
            alo = cdxt_adyt1 - ahi;
            _0 = alo * blo - (_j - ahi * bhi - alo * bhi - ahi * blo);
            _k = _i + _0;
            bvirt = _k - _i;
            u$1[1] = _i - (_k - bvirt) + (_0 - bvirt);
            u3 = _j + _k;
            u$1[2] = _k - (u3 - _j);
            u$1[3] = u3;
            finlength = fast_expansion_sum_zeroelim(finlength, finnow, 4, u$1, finother);
            finswap = finnow; finnow = finother; finother = finswap;
            if (bdztail !== 0) {
                c = splitter * bdztail;
                bhi = c - (c - bdztail);
                blo = bdztail - bhi;
                _i = cdxt_adyt0 * bdztail;
                c = splitter * cdxt_adyt0;
                ahi = c - (c - cdxt_adyt0);
                alo = cdxt_adyt0 - ahi;
                u$1[0] = alo * blo - (_i - ahi * bhi - alo * bhi - ahi * blo);
                _j = cdxt_adyt1 * bdztail;
                c = splitter * cdxt_adyt1;
                ahi = c - (c - cdxt_adyt1);
                alo = cdxt_adyt1 - ahi;
                _0 = alo * blo - (_j - ahi * bhi - alo * bhi - ahi * blo);
                _k = _i + _0;
                bvirt = _k - _i;
                u$1[1] = _i - (_k - bvirt) + (_0 - bvirt);
                u3 = _j + _k;
                u$1[2] = _k - (u3 - _j);
                u$1[3] = u3;
                finlength = fast_expansion_sum_zeroelim(finlength, finnow, 4, u$1, finother);
                finswap = finnow; finnow = finother; finother = finswap;
            }
        }
        if (bdytail !== 0) {
            negate = -cdxtail;
            cdxt_bdyt1 = negate * bdytail;
            c = splitter * negate;
            ahi = c - (c - negate);
            alo = negate - ahi;
            c = splitter * bdytail;
            bhi = c - (c - bdytail);
            blo = bdytail - bhi;
            cdxt_bdyt0 = alo * blo - (cdxt_bdyt1 - ahi * bhi - alo * bhi - ahi * blo);
            c = splitter * adz;
            bhi = c - (c - adz);
            blo = adz - bhi;
            _i = cdxt_bdyt0 * adz;
            c = splitter * cdxt_bdyt0;
            ahi = c - (c - cdxt_bdyt0);
            alo = cdxt_bdyt0 - ahi;
            u$1[0] = alo * blo - (_i - ahi * bhi - alo * bhi - ahi * blo);
            _j = cdxt_bdyt1 * adz;
            c = splitter * cdxt_bdyt1;
            ahi = c - (c - cdxt_bdyt1);
            alo = cdxt_bdyt1 - ahi;
            _0 = alo * blo - (_j - ahi * bhi - alo * bhi - ahi * blo);
            _k = _i + _0;
            bvirt = _k - _i;
            u$1[1] = _i - (_k - bvirt) + (_0 - bvirt);
            u3 = _j + _k;
            u$1[2] = _k - (u3 - _j);
            u$1[3] = u3;
            finlength = fast_expansion_sum_zeroelim(finlength, finnow, 4, u$1, finother);
            finswap = finnow; finnow = finother; finother = finswap;
            if (adztail !== 0) {
                c = splitter * adztail;
                bhi = c - (c - adztail);
                blo = adztail - bhi;
                _i = cdxt_bdyt0 * adztail;
                c = splitter * cdxt_bdyt0;
                ahi = c - (c - cdxt_bdyt0);
                alo = cdxt_bdyt0 - ahi;
                u$1[0] = alo * blo - (_i - ahi * bhi - alo * bhi - ahi * blo);
                _j = cdxt_bdyt1 * adztail;
                c = splitter * cdxt_bdyt1;
                ahi = c - (c - cdxt_bdyt1);
                alo = cdxt_bdyt1 - ahi;
                _0 = alo * blo - (_j - ahi * bhi - alo * bhi - ahi * blo);
                _k = _i + _0;
                bvirt = _k - _i;
                u$1[1] = _i - (_k - bvirt) + (_0 - bvirt);
                u3 = _j + _k;
                u$1[2] = _k - (u3 - _j);
                u$1[3] = u3;
                finlength = fast_expansion_sum_zeroelim(finlength, finnow, 4, u$1, finother);
                finswap = finnow; finnow = finother; finother = finswap;
            }
        }
    }

    if (adztail !== 0) {
        wlength = scale_expansion_zeroelim(bctlen, bct, adztail, w);
        finlength = fast_expansion_sum_zeroelim(finlength, finnow, wlength, w, finother);
        finswap = finnow; finnow = finother; finother = finswap;
    }
    if (bdztail !== 0) {
        wlength = scale_expansion_zeroelim(catlen, cat, bdztail, w);
        finlength = fast_expansion_sum_zeroelim(finlength, finnow, wlength, w, finother);
        finswap = finnow; finnow = finother; finother = finswap;
    }
    if (cdztail !== 0) {
        wlength = scale_expansion_zeroelim(abtlen, abt, cdztail, w);
        finlength = fast_expansion_sum_zeroelim(finlength, finnow, wlength, w, finother);
        finswap = finnow; finnow = finother; finother = finswap;
    }

    return finnow[finlength - 1];
}

function orient3d(ax, ay, az, bx, by, bz, cx, cy, cz, dx, dy, dz) {
    const adx = ax - dx;
    const bdx = bx - dx;
    const cdx = cx - dx;
    const ady = ay - dy;
    const bdy = by - dy;
    const cdy = cy - dy;
    const adz = az - dz;
    const bdz = bz - dz;
    const cdz = cz - dz;

    const bdxcdy = bdx * cdy;
    const cdxbdy = cdx * bdy;

    const cdxady = cdx * ady;
    const adxcdy = adx * cdy;

    const adxbdy = adx * bdy;
    const bdxady = bdx * ady;

    const det =
        adz * (bdxcdy - cdxbdy) +
        bdz * (cdxady - adxcdy) +
        cdz * (adxbdy - bdxady);

    const permanent =
        (Math.abs(bdxcdy) + Math.abs(cdxbdy)) * Math.abs(adz) +
        (Math.abs(cdxady) + Math.abs(adxcdy)) * Math.abs(bdz) +
        (Math.abs(adxbdy) + Math.abs(bdxady)) * Math.abs(cdz);

    const errbound = o3derrboundA * permanent;
    if (det > errbound || -det > errbound) {
        return det;
    }

    return orient3dadapt(ax, ay, az, bx, by, bz, cx, cy, cz, dx, dy, dz, permanent);
}

function orient3dfast(ax, ay, az, bx, by, bz, cx, cy, cz, dx, dy, dz) {
    const adx = ax - dx;
    const bdx = bx - dx;
    const cdx = cx - dx;
    const ady = ay - dy;
    const bdy = by - dy;
    const cdy = cy - dy;
    const adz = az - dz;
    const bdz = bz - dz;
    const cdz = cz - dz;

    return adx * (bdy * cdz - bdz * cdy) +
        bdx * (cdy * adz - cdz * ady) +
        cdx * (ady * bdz - adz * bdy);
}

const iccerrboundA = (10 + 96 * epsilon) * epsilon;
const iccerrboundB = (4 + 48 * epsilon) * epsilon;
const iccerrboundC = (44 + 576 * epsilon) * epsilon * epsilon;

const bc$1 = vec(4);
const ca$1 = vec(4);
const ab$1 = vec(4);
const axbc = vec(8);
const axxbc = vec(16);
const aybc = vec(8);
const ayybc = vec(16);
const adet$1 = vec(32);
const bxca = vec(8);
const bxxca = vec(16);
const byca = vec(8);
const byyca = vec(16);
const bdet$1 = vec(32);
const cxab = vec(8);
const cxxab = vec(16);
const cyab = vec(8);
const cyyab = vec(16);
const cdet$1 = vec(32);
const abdet$1 = vec(64);
const fin1$1 = vec(1152);
const fin2$1 = vec(1152);
const aa = vec(4);
const bb = vec(4);
const cc = vec(4);
const u$2 = vec(4);
const v$1 = vec(4);
const temp8 = vec(8);
const temp16a = vec(16);
const temp16b = vec(16);
const temp16c = vec(16);
const temp32a = vec(32);
const temp32b = vec(32);
const temp48 = vec(48);
const temp64 = vec(64);
const axtbb = vec(8);
const axtcc = vec(8);
const aytbb = vec(8);
const aytcc = vec(8);
const bxtaa = vec(8);
const bxtcc = vec(8);
const bytaa = vec(8);
const bytcc = vec(8);
const cxtaa = vec(8);
const cxtbb = vec(8);
const cytaa = vec(8);
const cytbb = vec(8);
const axtbc = vec(8);
const aytbc = vec(8);
const bxtca = vec(8);
const bytca = vec(8);
const cxtab = vec(8);
const cytab = vec(8);
const axtbct = vec(16);
const aytbct = vec(16);
const bxtcat = vec(16);
const bytcat = vec(16);
const cxtabt = vec(16);
const cytabt = vec(16);
const axtbctt = vec(8);
const aytbctt = vec(8);
const bxtcatt = vec(8);
const bytcatt = vec(8);
const cxtabtt = vec(8);
const cytabtt = vec(8);
const abt$1 = vec(8);
const bct$1 = vec(8);
const cat$1 = vec(8);
const abtt = vec(4);
const bctt = vec(4);
const catt = vec(4);

function incircleadapt(ax, ay, bx, by, cx, cy, dx, dy, permanent) {
    let bdxcdy1, cdxbdy1, cdxady1, adxcdy1, adxbdy1, bdxady1;
    let bdxcdy0, cdxbdy0, cdxady0, adxcdy0, adxbdy0, bdxady0;
    let bc3, ca3, ab3;
    let finnow, finother, finswap, finlength;

    let adxtail, bdxtail, cdxtail, adytail, bdytail, cdytail;
    let adxadx1, adyady1, bdxbdx1, bdybdy1, cdxcdx1, cdycdy1;
    let adxadx0, adyady0, bdxbdx0, bdybdy0, cdxcdx0, cdycdy0;
    let aa3, bb3, cc3;
    let ti1, tj1;
    let ti0, tj0;
    let u3, v3;
    let temp8len, temp16alen, temp16blen, temp16clen;
    let temp32alen, temp32blen, temp48len, temp64len;
    let axtbblen, axtcclen, aytbblen, aytcclen;
    let bxtaalen, bxtcclen, bytaalen, bytcclen;
    let cxtaalen, cxtbblen, cytaalen, cytbblen;
    let axtbclen, aytbclen, bxtcalen, bytcalen, cxtablen, cytablen;
    let axtbctlen, aytbctlen, bxtcatlen, bytcatlen, cxtabtlen, cytabtlen;
    let axtbcttlen, aytbcttlen, bxtcattlen, bytcattlen, cxtabttlen, cytabttlen;
    let abtlen, bctlen, catlen;
    let abttlen, bcttlen, cattlen;
    let abtt3, bctt3, catt3;
    let negate;

    let bvirt, c, ahi, alo, bhi, blo, _i, _j, _0;

    const adx = ax - dx;
    const bdx = bx - dx;
    const cdx = cx - dx;
    const ady = ay - dy;
    const bdy = by - dy;
    const cdy = cy - dy;

    bdxcdy1 = bdx * cdy;
    c = splitter * bdx;
    ahi = c - (c - bdx);
    alo = bdx - ahi;
    c = splitter * cdy;
    bhi = c - (c - cdy);
    blo = cdy - bhi;
    bdxcdy0 = alo * blo - (bdxcdy1 - ahi * bhi - alo * bhi - ahi * blo);
    cdxbdy1 = cdx * bdy;
    c = splitter * cdx;
    ahi = c - (c - cdx);
    alo = cdx - ahi;
    c = splitter * bdy;
    bhi = c - (c - bdy);
    blo = bdy - bhi;
    cdxbdy0 = alo * blo - (cdxbdy1 - ahi * bhi - alo * bhi - ahi * blo);
    _i = bdxcdy0 - cdxbdy0;
    bvirt = bdxcdy0 - _i;
    bc$1[0] = bdxcdy0 - (_i + bvirt) + (bvirt - cdxbdy0);
    _j = bdxcdy1 + _i;
    bvirt = _j - bdxcdy1;
    _0 = bdxcdy1 - (_j - bvirt) + (_i - bvirt);
    _i = _0 - cdxbdy1;
    bvirt = _0 - _i;
    bc$1[1] = _0 - (_i + bvirt) + (bvirt - cdxbdy1);
    bc3 = _j + _i;
    bvirt = bc3 - _j;
    bc$1[2] = _j - (bc3 - bvirt) + (_i - bvirt);
    bc$1[3] = bc3;
    const axbclen = scale_expansion_zeroelim(4, bc$1, adx, axbc);
    const axxbclen = scale_expansion_zeroelim(axbclen, axbc, adx, axxbc);
    const aybclen = scale_expansion_zeroelim(4, bc$1, ady, aybc);
    const ayybclen = scale_expansion_zeroelim(aybclen, aybc, ady, ayybc);
    const alen = fast_expansion_sum_zeroelim(axxbclen, axxbc, ayybclen, ayybc, adet$1);

    cdxady1 = cdx * ady;
    c = splitter * cdx;
    ahi = c - (c - cdx);
    alo = cdx - ahi;
    c = splitter * ady;
    bhi = c - (c - ady);
    blo = ady - bhi;
    cdxady0 = alo * blo - (cdxady1 - ahi * bhi - alo * bhi - ahi * blo);
    adxcdy1 = adx * cdy;
    c = splitter * adx;
    ahi = c - (c - adx);
    alo = adx - ahi;
    c = splitter * cdy;
    bhi = c - (c - cdy);
    blo = cdy - bhi;
    adxcdy0 = alo * blo - (adxcdy1 - ahi * bhi - alo * bhi - ahi * blo);
    _i = cdxady0 - adxcdy0;
    bvirt = cdxady0 - _i;
    ca$1[0] = cdxady0 - (_i + bvirt) + (bvirt - adxcdy0);
    _j = cdxady1 + _i;
    bvirt = _j - cdxady1;
    _0 = cdxady1 - (_j - bvirt) + (_i - bvirt);
    _i = _0 - adxcdy1;
    bvirt = _0 - _i;
    ca$1[1] = _0 - (_i + bvirt) + (bvirt - adxcdy1);
    ca3 = _j + _i;
    bvirt = ca3 - _j;
    ca$1[2] = _j - (ca3 - bvirt) + (_i - bvirt);
    ca$1[3] = ca3;
    const bxcalen = scale_expansion_zeroelim(4, ca$1, bdx, bxca);
    const bxxcalen = scale_expansion_zeroelim(bxcalen, bxca, bdx, bxxca);
    const bycalen = scale_expansion_zeroelim(4, ca$1, bdy, byca);
    const byycalen = scale_expansion_zeroelim(bycalen, byca, bdy, byyca);
    const blen = fast_expansion_sum_zeroelim(bxxcalen, bxxca, byycalen, byyca, bdet$1);

    adxbdy1 = adx * bdy;
    c = splitter * adx;
    ahi = c - (c - adx);
    alo = adx - ahi;
    c = splitter * bdy;
    bhi = c - (c - bdy);
    blo = bdy - bhi;
    adxbdy0 = alo * blo - (adxbdy1 - ahi * bhi - alo * bhi - ahi * blo);
    bdxady1 = bdx * ady;
    c = splitter * bdx;
    ahi = c - (c - bdx);
    alo = bdx - ahi;
    c = splitter * ady;
    bhi = c - (c - ady);
    blo = ady - bhi;
    bdxady0 = alo * blo - (bdxady1 - ahi * bhi - alo * bhi - ahi * blo);
    _i = adxbdy0 - bdxady0;
    bvirt = adxbdy0 - _i;
    ab$1[0] = adxbdy0 - (_i + bvirt) + (bvirt - bdxady0);
    _j = adxbdy1 + _i;
    bvirt = _j - adxbdy1;
    _0 = adxbdy1 - (_j - bvirt) + (_i - bvirt);
    _i = _0 - bdxady1;
    bvirt = _0 - _i;
    ab$1[1] = _0 - (_i + bvirt) + (bvirt - bdxady1);
    ab3 = _j + _i;
    bvirt = ab3 - _j;
    ab$1[2] = _j - (ab3 - bvirt) + (_i - bvirt);
    ab$1[3] = ab3;
    const cxablen = scale_expansion_zeroelim(4, ab$1, cdx, cxab);
    const cxxablen = scale_expansion_zeroelim(cxablen, cxab, cdx, cxxab);
    const cyablen = scale_expansion_zeroelim(4, ab$1, cdy, cyab);
    const cyyablen = scale_expansion_zeroelim(cyablen, cyab, cdy, cyyab);
    const clen = fast_expansion_sum_zeroelim(cxxablen, cxxab, cyyablen, cyyab, cdet$1);

    const ablen = fast_expansion_sum_zeroelim(alen, adet$1, blen, bdet$1, abdet$1);
    finlength = fast_expansion_sum_zeroelim(ablen, abdet$1, clen, cdet$1, fin1$1);

    let det = estimate(finlength, fin1$1);
    let errbound = iccerrboundB * permanent;
    if (det >= errbound || -det >= errbound) {
        return det;
    }

    bvirt = ax - adx;
    adxtail = ax - (adx + bvirt) + (bvirt - dx);
    bvirt = ay - ady;
    adytail = ay - (ady + bvirt) + (bvirt - dy);
    bvirt = bx - bdx;
    bdxtail = bx - (bdx + bvirt) + (bvirt - dx);
    bvirt = by - bdy;
    bdytail = by - (bdy + bvirt) + (bvirt - dy);
    bvirt = cx - cdx;
    cdxtail = cx - (cdx + bvirt) + (bvirt - dx);
    bvirt = cy - cdy;
    cdytail = cy - (cdy + bvirt) + (bvirt - dy);
    if (adxtail === 0 && bdxtail === 0 && cdxtail === 0 && adytail === 0 && bdytail === 0 && cdytail === 0) {
        return det;
    }

    errbound = iccerrboundC * permanent + resulterrbound * Math.abs(det);
    det += ((adx * adx + ady * ady) * ((bdx * cdytail + cdy * bdxtail) - (bdy * cdxtail + cdx * bdytail)) +
        2 * (adx * adxtail + ady * adytail) * (bdx * cdy - bdy * cdx)) +
        ((bdx * bdx + bdy * bdy) * ((cdx * adytail + ady * cdxtail) - (cdy * adxtail + adx * cdytail)) +
        2 * (bdx * bdxtail + bdy * bdytail) * (cdx * ady - cdy * adx)) +
        ((cdx * cdx + cdy * cdy) * ((adx * bdytail + bdy * adxtail) - (ady * bdxtail + bdx * adytail)) +
        2 * (cdx * cdxtail + cdy * cdytail) * (adx * bdy - ady * bdx));

    if (det >= errbound || -det >= errbound) {
        return det;
    }

    finnow = fin1$1;
    finother = fin2$1;

    if (bdxtail !== 0 || bdytail !== 0 || cdxtail !== 0 || cdytail !== 0) {
        adxadx1 = adx * adx;
        c = splitter * adx;
        ahi = c - (c - adx);
        alo = adx - ahi;
        adxadx0 = alo * alo - (adxadx1 - ahi * ahi - (ahi + ahi) * alo);
        adyady1 = ady * ady;
        c = splitter * ady;
        ahi = c - (c - ady);
        alo = ady - ahi;
        adyady0 = alo * alo - (adyady1 - ahi * ahi - (ahi + ahi) * alo);
        _i = adxadx0 + adyady0;
        bvirt = _i - adxadx0;
        aa[0] = adxadx0 - (_i - bvirt) + (adyady0 - bvirt);
        _j = adxadx1 + _i;
        bvirt = _j - adxadx1;
        _0 = adxadx1 - (_j - bvirt) + (_i - bvirt);
        _i = _0 + adyady1;
        bvirt = _i - _0;
        aa[1] = _0 - (_i - bvirt) + (adyady1 - bvirt);
        aa3 = _j + _i;
        bvirt = aa3 - _j;
        aa[2] = _j - (aa3 - bvirt) + (_i - bvirt);
        aa[3] = aa3;
    }
    if (cdxtail !== 0 || cdytail !== 0 || adxtail !== 0 || adytail !== 0) {
        bdxbdx1 = bdx * bdx;
        c = splitter * bdx;
        ahi = c - (c - bdx);
        alo = bdx - ahi;
        bdxbdx0 = alo * alo - (bdxbdx1 - ahi * ahi - (ahi + ahi) * alo);
        bdybdy1 = bdy * bdy;
        c = splitter * bdy;
        ahi = c - (c - bdy);
        alo = bdy - ahi;
        bdybdy0 = alo * alo - (bdybdy1 - ahi * ahi - (ahi + ahi) * alo);
        _i = bdxbdx0 + bdybdy0;
        bvirt = _i - bdxbdx0;
        bb[0] = bdxbdx0 - (_i - bvirt) + (bdybdy0 - bvirt);
        _j = bdxbdx1 + _i;
        bvirt = _j - bdxbdx1;
        _0 = bdxbdx1 - (_j - bvirt) + (_i - bvirt);
        _i = _0 + bdybdy1;
        bvirt = _i - _0;
        bb[1] = _0 - (_i - bvirt) + (bdybdy1 - bvirt);
        bb3 = _j + _i;
        bvirt = bb3 - _j;
        bb[2] = _j - (bb3 - bvirt) + (_i - bvirt);
        bb[3] = bb3;
    }
    if (adxtail !== 0 || adytail !== 0 || bdxtail !== 0 || bdytail !== 0) {
        cdxcdx1 = cdx * cdx;
        c = splitter * cdx;
        ahi = c - (c - cdx);
        alo = cdx - ahi;
        cdxcdx0 = alo * alo - (cdxcdx1 - ahi * ahi - (ahi + ahi) * alo);
        cdycdy1 = cdy * cdy;
        c = splitter * cdy;
        ahi = c - (c - cdy);
        alo = cdy - ahi;
        cdycdy0 = alo * alo - (cdycdy1 - ahi * ahi - (ahi + ahi) * alo);
        _i = cdxcdx0 + cdycdy0;
        bvirt = _i - cdxcdx0;
        cc[0] = cdxcdx0 - (_i - bvirt) + (cdycdy0 - bvirt);
        _j = cdxcdx1 + _i;
        bvirt = _j - cdxcdx1;
        _0 = cdxcdx1 - (_j - bvirt) + (_i - bvirt);
        _i = _0 + cdycdy1;
        bvirt = _i - _0;
        cc[1] = _0 - (_i - bvirt) + (cdycdy1 - bvirt);
        cc3 = _j + _i;
        bvirt = cc3 - _j;
        cc[2] = _j - (cc3 - bvirt) + (_i - bvirt);
        cc[3] = cc3;
    }

    if (adxtail !== 0) {
        axtbclen = scale_expansion_zeroelim(4, bc$1, adxtail, axtbc);
        temp16alen = scale_expansion_zeroelim(axtbclen, axtbc, 2 * adx, temp16a);

        axtcclen = scale_expansion_zeroelim(4, cc, adxtail, axtcc);
        temp16blen = scale_expansion_zeroelim(axtcclen, axtcc, bdy, temp16b);

        axtbblen = scale_expansion_zeroelim(4, bb, adxtail, axtbb);
        temp16clen = scale_expansion_zeroelim(axtbblen, axtbb, -cdy, temp16c);

        temp32alen = fast_expansion_sum_zeroelim(temp16alen, temp16a, temp16blen, temp16b, temp32a);
        temp48len = fast_expansion_sum_zeroelim(temp16clen, temp16c, temp32alen, temp32a, temp48);
        finlength = fast_expansion_sum_zeroelim(finlength, finnow, temp48len, temp48, finother);
        finswap = finnow; finnow = finother; finother = finswap;
    }
    if (adytail !== 0) {
        aytbclen = scale_expansion_zeroelim(4, bc$1, adytail, aytbc);
        temp16alen = scale_expansion_zeroelim(aytbclen, aytbc, 2 * ady, temp16a);

        aytbblen = scale_expansion_zeroelim(4, bb, adytail, aytbb);
        temp16blen = scale_expansion_zeroelim(aytbblen, aytbb, cdx, temp16b);

        aytcclen = scale_expansion_zeroelim(4, cc, adytail, aytcc);
        temp16clen = scale_expansion_zeroelim(aytcclen, aytcc, -bdx, temp16c);

        temp32alen = fast_expansion_sum_zeroelim(temp16alen, temp16a, temp16blen, temp16b, temp32a);
        temp48len = fast_expansion_sum_zeroelim(temp16clen, temp16c, temp32alen, temp32a, temp48);
        finlength = fast_expansion_sum_zeroelim(finlength, finnow, temp48len, temp48, finother);
        finswap = finnow; finnow = finother; finother = finswap;
    }
    if (bdxtail !== 0) {
        bxtcalen = scale_expansion_zeroelim(4, ca$1, bdxtail, bxtca);
        temp16alen = scale_expansion_zeroelim(bxtcalen, bxtca, 2 * bdx, temp16a);

        bxtaalen = scale_expansion_zeroelim(4, aa, bdxtail, bxtaa);
        temp16blen = scale_expansion_zeroelim(bxtaalen, bxtaa, cdy, temp16b);

        bxtcclen = scale_expansion_zeroelim(4, cc, bdxtail, bxtcc);
        temp16clen = scale_expansion_zeroelim(bxtcclen, bxtcc, -ady, temp16c);

        temp32alen = fast_expansion_sum_zeroelim(temp16alen, temp16a, temp16blen, temp16b, temp32a);
        temp48len = fast_expansion_sum_zeroelim(temp16clen, temp16c, temp32alen, temp32a, temp48);
        finlength = fast_expansion_sum_zeroelim(finlength, finnow, temp48len, temp48, finother);
        finswap = finnow; finnow = finother; finother = finswap;
    }
    if (bdytail !== 0) {
        bytcalen = scale_expansion_zeroelim(4, ca$1, bdytail, bytca);
        temp16alen = scale_expansion_zeroelim(bytcalen, bytca, 2 * bdy, temp16a);

        bytcclen = scale_expansion_zeroelim(4, cc, bdytail, bytcc);
        temp16blen = scale_expansion_zeroelim(bytcclen, bytcc, adx, temp16b);

        bytaalen = scale_expansion_zeroelim(4, aa, bdytail, bytaa);
        temp16clen = scale_expansion_zeroelim(bytaalen, bytaa, -cdx, temp16c);

        temp32alen = fast_expansion_sum_zeroelim(temp16alen, temp16a, temp16blen, temp16b, temp32a);
        temp48len = fast_expansion_sum_zeroelim(temp16clen, temp16c, temp32alen, temp32a, temp48);
        finlength = fast_expansion_sum_zeroelim(finlength, finnow, temp48len, temp48, finother);
        finswap = finnow; finnow = finother; finother = finswap;
    }
    if (cdxtail !== 0) {
        cxtablen = scale_expansion_zeroelim(4, ab$1, cdxtail, cxtab);
        temp16alen = scale_expansion_zeroelim(cxtablen, cxtab, 2 * cdx, temp16a);

        cxtbblen = scale_expansion_zeroelim(4, bb, cdxtail, cxtbb);
        temp16blen = scale_expansion_zeroelim(cxtbblen, cxtbb, ady, temp16b);

        cxtaalen = scale_expansion_zeroelim(4, aa, cdxtail, cxtaa);
        temp16clen = scale_expansion_zeroelim(cxtaalen, cxtaa, -bdy, temp16c);

        temp32alen = fast_expansion_sum_zeroelim(temp16alen, temp16a, temp16blen, temp16b, temp32a);
        temp48len = fast_expansion_sum_zeroelim(temp16clen, temp16c, temp32alen, temp32a, temp48);
        finlength = fast_expansion_sum_zeroelim(finlength, finnow, temp48len, temp48, finother);
        finswap = finnow; finnow = finother; finother = finswap;
    }
    if (cdytail !== 0) {
        cytablen = scale_expansion_zeroelim(4, ab$1, cdytail, cytab);
        temp16alen = scale_expansion_zeroelim(cytablen, cytab, 2 * cdy, temp16a);

        cytaalen = scale_expansion_zeroelim(4, aa, cdytail, cytaa);
        temp16blen = scale_expansion_zeroelim(cytaalen, cytaa, bdx, temp16b);

        cytbblen = scale_expansion_zeroelim(4, bb, cdytail, cytbb);
        temp16clen = scale_expansion_zeroelim(cytbblen, cytbb, -adx, temp16c);

        temp32alen = fast_expansion_sum_zeroelim(temp16alen, temp16a, temp16blen, temp16b, temp32a);
        temp48len = fast_expansion_sum_zeroelim(temp16clen, temp16c, temp32alen, temp32a, temp48);
        finlength = fast_expansion_sum_zeroelim(finlength, finnow, temp48len, temp48, finother);
        finswap = finnow; finnow = finother; finother = finswap;
    }

    if (adxtail !== 0 || adytail !== 0) {
        if (bdxtail !== 0 || bdytail !== 0 || cdxtail !== 0 || cdytail !== 0) {
            ti1 = bdxtail * cdy;
            c = splitter * bdxtail;
            ahi = c - (c - bdxtail);
            alo = bdxtail - ahi;
            c = splitter * cdy;
            bhi = c - (c - cdy);
            blo = cdy - bhi;
            ti0 = alo * blo - (ti1 - ahi * bhi - alo * bhi - ahi * blo);
            tj1 = bdx * cdytail;
            c = splitter * bdx;
            ahi = c - (c - bdx);
            alo = bdx - ahi;
            c = splitter * cdytail;
            bhi = c - (c - cdytail);
            blo = cdytail - bhi;
            tj0 = alo * blo - (tj1 - ahi * bhi - alo * bhi - ahi * blo);
            _i = ti0 + tj0;
            bvirt = _i - ti0;
            u$2[0] = ti0 - (_i - bvirt) + (tj0 - bvirt);
            _j = ti1 + _i;
            bvirt = _j - ti1;
            _0 = ti1 - (_j - bvirt) + (_i - bvirt);
            _i = _0 + tj1;
            bvirt = _i - _0;
            u$2[1] = _0 - (_i - bvirt) + (tj1 - bvirt);
            u3 = _j + _i;
            bvirt = u3 - _j;
            u$2[2] = _j - (u3 - bvirt) + (_i - bvirt);
            u$2[3] = u3;
            negate = -bdy;
            ti1 = cdxtail * negate;
            c = splitter * cdxtail;
            ahi = c - (c - cdxtail);
            alo = cdxtail - ahi;
            c = splitter * negate;
            bhi = c - (c - negate);
            blo = negate - bhi;
            ti0 = alo * blo - (ti1 - ahi * bhi - alo * bhi - ahi * blo);
            negate = -bdytail;
            tj1 = cdx * negate;
            c = splitter * cdx;
            ahi = c - (c - cdx);
            alo = cdx - ahi;
            c = splitter * negate;
            bhi = c - (c - negate);
            blo = negate - bhi;
            tj0 = alo * blo - (tj1 - ahi * bhi - alo * bhi - ahi * blo);
            _i = ti0 + tj0;
            bvirt = _i - ti0;
            v$1[0] = ti0 - (_i - bvirt) + (tj0 - bvirt);
            _j = ti1 + _i;
            bvirt = _j - ti1;
            _0 = ti1 - (_j - bvirt) + (_i - bvirt);
            _i = _0 + tj1;
            bvirt = _i - _0;
            v$1[1] = _0 - (_i - bvirt) + (tj1 - bvirt);
            v3 = _j + _i;
            bvirt = v3 - _j;
            v$1[2] = _j - (v3 - bvirt) + (_i - bvirt);
            v$1[3] = v3;
            bctlen = fast_expansion_sum_zeroelim(4, u$2, 4, v$1, bct$1);

            ti1 = bdxtail * cdytail;
            c = splitter * bdxtail;
            ahi = c - (c - bdxtail);
            alo = bdxtail - ahi;
            c = splitter * cdytail;
            bhi = c - (c - cdytail);
            blo = cdytail - bhi;
            ti0 = alo * blo - (ti1 - ahi * bhi - alo * bhi - ahi * blo);
            tj1 = cdxtail * bdytail;
            c = splitter * cdxtail;
            ahi = c - (c - cdxtail);
            alo = cdxtail - ahi;
            c = splitter * bdytail;
            bhi = c - (c - bdytail);
            blo = bdytail - bhi;
            tj0 = alo * blo - (tj1 - ahi * bhi - alo * bhi - ahi * blo);
            _i = ti0 - tj0;
            bvirt = ti0 - _i;
            bctt[0] = ti0 - (_i + bvirt) + (bvirt - tj0);
            _j = ti1 + _i;
            bvirt = _j - ti1;
            _0 = ti1 - (_j - bvirt) + (_i - bvirt);
            _i = _0 - tj1;
            bvirt = _0 - _i;
            bctt[1] = _0 - (_i + bvirt) + (bvirt - tj1);
            bctt3 = _j + _i;
            bvirt = bctt3 - _j;
            bctt[2] = _j - (bctt3 - bvirt) + (_i - bvirt);
            bctt[3] = bctt3;
            bcttlen = 4;
        } else {
            bct$1[0] = 0;
            bctlen = 1;
            bctt[0] = 0;
            bcttlen = 1;
        }

        if (adxtail !== 0) {
            temp16alen = scale_expansion_zeroelim(axtbclen, axtbc, adxtail, temp16a);
            axtbctlen = scale_expansion_zeroelim(bctlen, bct$1, adxtail, axtbct);
            temp32alen = scale_expansion_zeroelim(axtbctlen, axtbct, 2 * adx, temp32a);
            temp48len = fast_expansion_sum_zeroelim(temp16alen, temp16a, temp32alen, temp32a, temp48);
            finlength = fast_expansion_sum_zeroelim(finlength, finnow, temp48len, temp48, finother);
            finswap = finnow; finnow = finother; finother = finswap;

            if (bdytail !== 0) {
                temp8len = scale_expansion_zeroelim(4, cc, adxtail, temp8);
                temp16alen = scale_expansion_zeroelim(temp8len, temp8, bdytail, temp16a);
                finlength = fast_expansion_sum_zeroelim(finlength, finnow, temp16alen, temp16a, finother);
                finswap = finnow; finnow = finother; finother = finswap;
            }
            if (cdytail !== 0) {
                temp8len = scale_expansion_zeroelim(4, bb, -adxtail, temp8);
                temp16alen = scale_expansion_zeroelim(temp8len, temp8, cdytail, temp16a);
                finlength = fast_expansion_sum_zeroelim(finlength, finnow, temp16alen, temp16a, finother);
                finswap = finnow; finnow = finother; finother = finswap;
            }

            temp32alen = scale_expansion_zeroelim(axtbctlen, axtbct, adxtail, temp32a);
            axtbcttlen = scale_expansion_zeroelim(bcttlen, bctt, adxtail, axtbctt);
            temp16alen = scale_expansion_zeroelim(axtbcttlen, axtbctt, 2 * adx, temp16a);
            temp16blen = scale_expansion_zeroelim(axtbcttlen, axtbctt, adxtail, temp16b);
            temp32blen = fast_expansion_sum_zeroelim(temp16alen, temp16a, temp16blen, temp16b, temp32b);
            temp64len = fast_expansion_sum_zeroelim(temp32alen, temp32a, temp32blen, temp32b, temp64);
            finlength = fast_expansion_sum_zeroelim(finlength, finnow, temp64len, temp64, finother);
            finswap = finnow; finnow = finother; finother = finswap;
        }

        if (adytail !== 0) {
            temp16alen = scale_expansion_zeroelim(aytbclen, aytbc, adytail, temp16a);
            aytbctlen = scale_expansion_zeroelim(bctlen, bct$1, adytail, aytbct);
            temp32alen = scale_expansion_zeroelim(aytbctlen, aytbct, 2 * ady, temp32a);
            temp48len = fast_expansion_sum_zeroelim(temp16alen, temp16a, temp32alen, temp32a, temp48);
            finlength = fast_expansion_sum_zeroelim(finlength, finnow, temp48len, temp48, finother);
            finswap = finnow; finnow = finother; finother = finswap;

            temp32alen = scale_expansion_zeroelim(aytbctlen, aytbct, adytail, temp32a);
            aytbcttlen = scale_expansion_zeroelim(bcttlen, bctt, adytail, aytbctt);
            temp16alen = scale_expansion_zeroelim(aytbcttlen, aytbctt, 2 * ady, temp16a);
            temp16blen = scale_expansion_zeroelim(aytbcttlen, aytbctt, adytail, temp16b);
            temp32blen = fast_expansion_sum_zeroelim(temp16alen, temp16a, temp16blen, temp16b, temp32b);
            temp64len = fast_expansion_sum_zeroelim(temp32alen, temp32a, temp32blen, temp32b, temp64);
            finlength = fast_expansion_sum_zeroelim(finlength, finnow, temp64len, temp64, finother);
            finswap = finnow; finnow = finother; finother = finswap;
        }
    }
    if (bdxtail !== 0 || bdytail !== 0) {
        if (cdxtail !== 0 || cdytail !== 0 || adxtail !== 0 || adytail !== 0) {
            ti1 = cdxtail * ady;
            c = splitter * cdxtail;
            ahi = c - (c - cdxtail);
            alo = cdxtail - ahi;
            c = splitter * ady;
            bhi = c - (c - ady);
            blo = ady - bhi;
            ti0 = alo * blo - (ti1 - ahi * bhi - alo * bhi - ahi * blo);
            tj1 = cdx * adytail;
            c = splitter * cdx;
            ahi = c - (c - cdx);
            alo = cdx - ahi;
            c = splitter * adytail;
            bhi = c - (c - adytail);
            blo = adytail - bhi;
            tj0 = alo * blo - (tj1 - ahi * bhi - alo * bhi - ahi * blo);
            _i = ti0 + tj0;
            bvirt = _i - ti0;
            u$2[0] = ti0 - (_i - bvirt) + (tj0 - bvirt);
            _j = ti1 + _i;
            bvirt = _j - ti1;
            _0 = ti1 - (_j - bvirt) + (_i - bvirt);
            _i = _0 + tj1;
            bvirt = _i - _0;
            u$2[1] = _0 - (_i - bvirt) + (tj1 - bvirt);
            u3 = _j + _i;
            bvirt = u3 - _j;
            u$2[2] = _j - (u3 - bvirt) + (_i - bvirt);
            u$2[3] = u3;
            negate = -cdy;
            ti1 = adxtail * negate;
            c = splitter * adxtail;
            ahi = c - (c - adxtail);
            alo = adxtail - ahi;
            c = splitter * negate;
            bhi = c - (c - negate);
            blo = negate - bhi;
            ti0 = alo * blo - (ti1 - ahi * bhi - alo * bhi - ahi * blo);
            negate = -cdytail;
            tj1 = adx * negate;
            c = splitter * adx;
            ahi = c - (c - adx);
            alo = adx - ahi;
            c = splitter * negate;
            bhi = c - (c - negate);
            blo = negate - bhi;
            tj0 = alo * blo - (tj1 - ahi * bhi - alo * bhi - ahi * blo);
            _i = ti0 + tj0;
            bvirt = _i - ti0;
            v$1[0] = ti0 - (_i - bvirt) + (tj0 - bvirt);
            _j = ti1 + _i;
            bvirt = _j - ti1;
            _0 = ti1 - (_j - bvirt) + (_i - bvirt);
            _i = _0 + tj1;
            bvirt = _i - _0;
            v$1[1] = _0 - (_i - bvirt) + (tj1 - bvirt);
            v3 = _j + _i;
            bvirt = v3 - _j;
            v$1[2] = _j - (v3 - bvirt) + (_i - bvirt);
            v$1[3] = v3;
            catlen = fast_expansion_sum_zeroelim(4, u$2, 4, v$1, cat$1);

            ti1 = cdxtail * adytail;
            c = splitter * cdxtail;
            ahi = c - (c - cdxtail);
            alo = cdxtail - ahi;
            c = splitter * adytail;
            bhi = c - (c - adytail);
            blo = adytail - bhi;
            ti0 = alo * blo - (ti1 - ahi * bhi - alo * bhi - ahi * blo);
            tj1 = adxtail * cdytail;
            c = splitter * adxtail;
            ahi = c - (c - adxtail);
            alo = adxtail - ahi;
            c = splitter * cdytail;
            bhi = c - (c - cdytail);
            blo = cdytail - bhi;
            tj0 = alo * blo - (tj1 - ahi * bhi - alo * bhi - ahi * blo);
            _i = ti0 - tj0;
            bvirt = ti0 - _i;
            catt[0] = ti0 - (_i + bvirt) + (bvirt - tj0);
            _j = ti1 + _i;
            bvirt = _j - ti1;
            _0 = ti1 - (_j - bvirt) + (_i - bvirt);
            _i = _0 - tj1;
            bvirt = _0 - _i;
            catt[1] = _0 - (_i + bvirt) + (bvirt - tj1);
            catt3 = _j + _i;
            bvirt = catt3 - _j;
            catt[2] = _j - (catt3 - bvirt) + (_i - bvirt);
            catt[3] = catt3;
            cattlen = 4;
        } else {
            cat$1[0] = 0;
            catlen = 1;
            catt[0] = 0;
            cattlen = 1;
        }

        if (bdxtail !== 0) {
            temp16alen = scale_expansion_zeroelim(bxtcalen, bxtca, bdxtail, temp16a);
            bxtcatlen = scale_expansion_zeroelim(catlen, cat$1, bdxtail, bxtcat);
            temp32alen = scale_expansion_zeroelim(bxtcatlen, bxtcat, 2 * bdx, temp32a);
            temp48len = fast_expansion_sum_zeroelim(temp16alen, temp16a, temp32alen, temp32a, temp48);
            finlength = fast_expansion_sum_zeroelim(finlength, finnow, temp48len, temp48, finother);
            finswap = finnow; finnow = finother; finother = finswap;
            if (cdytail !== 0) {
                temp8len = scale_expansion_zeroelim(4, aa, bdxtail, temp8);
                temp16alen = scale_expansion_zeroelim(temp8len, temp8, cdytail, temp16a);
                finlength = fast_expansion_sum_zeroelim(finlength, finnow, temp16alen, temp16a, finother);
                finswap = finnow; finnow = finother; finother = finswap;
            }
            if (adytail !== 0) {
                temp8len = scale_expansion_zeroelim(4, cc, -bdxtail, temp8);
                temp16alen = scale_expansion_zeroelim(temp8len, temp8, adytail, temp16a);
                finlength = fast_expansion_sum_zeroelim(finlength, finnow, temp16alen, temp16a, finother);
                finswap = finnow; finnow = finother; finother = finswap;
            }

            temp32alen = scale_expansion_zeroelim(bxtcatlen, bxtcat, bdxtail, temp32a);
            bxtcattlen = scale_expansion_zeroelim(cattlen, catt, bdxtail, bxtcatt);
            temp16alen = scale_expansion_zeroelim(bxtcattlen, bxtcatt, 2 * bdx, temp16a);
            temp16blen = scale_expansion_zeroelim(bxtcattlen, bxtcatt, bdxtail, temp16b);
            temp32blen = fast_expansion_sum_zeroelim(temp16alen, temp16a, temp16blen, temp16b, temp32b);
            temp64len = fast_expansion_sum_zeroelim(temp32alen, temp32a, temp32blen, temp32b, temp64);
            finlength = fast_expansion_sum_zeroelim(finlength, finnow, temp64len, temp64, finother);
            finswap = finnow; finnow = finother; finother = finswap;
        }
        if (bdytail !== 0) {
            temp16alen = scale_expansion_zeroelim(bytcalen, bytca, bdytail, temp16a);
            bytcatlen = scale_expansion_zeroelim(catlen, cat$1, bdytail, bytcat);
            temp32alen = scale_expansion_zeroelim(bytcatlen, bytcat, 2 * bdy, temp32a);
            temp48len = fast_expansion_sum_zeroelim(temp16alen, temp16a, temp32alen, temp32a, temp48);
            finlength = fast_expansion_sum_zeroelim(finlength, finnow, temp48len, temp48, finother);
            finswap = finnow; finnow = finother; finother = finswap;

            temp32alen = scale_expansion_zeroelim(bytcatlen, bytcat, bdytail, temp32a);
            bytcattlen = scale_expansion_zeroelim(cattlen, catt, bdytail, bytcatt);
            temp16alen = scale_expansion_zeroelim(bytcattlen, bytcatt, 2 * bdy, temp16a);
            temp16blen = scale_expansion_zeroelim(bytcattlen, bytcatt, bdytail, temp16b);
            temp32blen = fast_expansion_sum_zeroelim(temp16alen, temp16a, temp16blen, temp16b, temp32b);
            temp64len = fast_expansion_sum_zeroelim(temp32alen, temp32a, temp32blen, temp32b, temp64);
            finlength = fast_expansion_sum_zeroelim(finlength, finnow, temp64len, temp64, finother);
            finswap = finnow; finnow = finother; finother = finswap;
        }
    }
    if (cdxtail !== 0 || cdytail !== 0) {
        if (adxtail !== 0 || adytail !== 0 || bdxtail !== 0 || bdytail !== 0) {
            ti1 = adxtail * bdy;
            c = splitter * adxtail;
            ahi = c - (c - adxtail);
            alo = adxtail - ahi;
            c = splitter * bdy;
            bhi = c - (c - bdy);
            blo = bdy - bhi;
            ti0 = alo * blo - (ti1 - ahi * bhi - alo * bhi - ahi * blo);
            tj1 = adx * bdytail;
            c = splitter * adx;
            ahi = c - (c - adx);
            alo = adx - ahi;
            c = splitter * bdytail;
            bhi = c - (c - bdytail);
            blo = bdytail - bhi;
            tj0 = alo * blo - (tj1 - ahi * bhi - alo * bhi - ahi * blo);
            _i = ti0 + tj0;
            bvirt = _i - ti0;
            u$2[0] = ti0 - (_i - bvirt) + (tj0 - bvirt);
            _j = ti1 + _i;
            bvirt = _j - ti1;
            _0 = ti1 - (_j - bvirt) + (_i - bvirt);
            _i = _0 + tj1;
            bvirt = _i - _0;
            u$2[1] = _0 - (_i - bvirt) + (tj1 - bvirt);
            u3 = _j + _i;
            bvirt = u3 - _j;
            u$2[2] = _j - (u3 - bvirt) + (_i - bvirt);
            u$2[3] = u3;
            negate = -ady;
            ti1 = bdxtail * negate;
            c = splitter * bdxtail;
            ahi = c - (c - bdxtail);
            alo = bdxtail - ahi;
            c = splitter * negate;
            bhi = c - (c - negate);
            blo = negate - bhi;
            ti0 = alo * blo - (ti1 - ahi * bhi - alo * bhi - ahi * blo);
            negate = -adytail;
            tj1 = bdx * negate;
            c = splitter * bdx;
            ahi = c - (c - bdx);
            alo = bdx - ahi;
            c = splitter * negate;
            bhi = c - (c - negate);
            blo = negate - bhi;
            tj0 = alo * blo - (tj1 - ahi * bhi - alo * bhi - ahi * blo);
            _i = ti0 + tj0;
            bvirt = _i - ti0;
            v$1[0] = ti0 - (_i - bvirt) + (tj0 - bvirt);
            _j = ti1 + _i;
            bvirt = _j - ti1;
            _0 = ti1 - (_j - bvirt) + (_i - bvirt);
            _i = _0 + tj1;
            bvirt = _i - _0;
            v$1[1] = _0 - (_i - bvirt) + (tj1 - bvirt);
            v3 = _j + _i;
            bvirt = v3 - _j;
            v$1[2] = _j - (v3 - bvirt) + (_i - bvirt);
            v$1[3] = v3;
            abtlen = fast_expansion_sum_zeroelim(4, u$2, 4, v$1, abt$1);

            ti1 = adxtail * bdytail;
            c = splitter * adxtail;
            ahi = c - (c - adxtail);
            alo = adxtail - ahi;
            c = splitter * bdytail;
            bhi = c - (c - bdytail);
            blo = bdytail - bhi;
            ti0 = alo * blo - (ti1 - ahi * bhi - alo * bhi - ahi * blo);
            tj1 = bdxtail * adytail;
            c = splitter * bdxtail;
            ahi = c - (c - bdxtail);
            alo = bdxtail - ahi;
            c = splitter * adytail;
            bhi = c - (c - adytail);
            blo = adytail - bhi;
            tj0 = alo * blo - (tj1 - ahi * bhi - alo * bhi - ahi * blo);
            _i = ti0 - tj0;
            bvirt = ti0 - _i;
            abtt[0] = ti0 - (_i + bvirt) + (bvirt - tj0);
            _j = ti1 + _i;
            bvirt = _j - ti1;
            _0 = ti1 - (_j - bvirt) + (_i - bvirt);
            _i = _0 - tj1;
            bvirt = _0 - _i;
            abtt[1] = _0 - (_i + bvirt) + (bvirt - tj1);
            abtt3 = _j + _i;
            bvirt = abtt3 - _j;
            abtt[2] = _j - (abtt3 - bvirt) + (_i - bvirt);
            abtt[3] = abtt3;
            abttlen = 4;
        } else {
            abt$1[0] = 0;
            abtlen = 1;
            abtt[0] = 0;
            abttlen = 1;
        }

        if (cdxtail !== 0) {
            temp16alen = scale_expansion_zeroelim(cxtablen, cxtab, cdxtail, temp16a);
            cxtabtlen = scale_expansion_zeroelim(abtlen, abt$1, cdxtail, cxtabt);
            temp32alen = scale_expansion_zeroelim(cxtabtlen, cxtabt, 2 * cdx, temp32a);
            temp48len = fast_expansion_sum_zeroelim(temp16alen, temp16a, temp32alen, temp32a, temp48);
            finlength = fast_expansion_sum_zeroelim(finlength, finnow, temp48len, temp48, finother);
            finswap = finnow; finnow = finother; finother = finswap;
            if (adytail !== 0) {
                temp8len = scale_expansion_zeroelim(4, bb, cdxtail, temp8);
                temp16alen = scale_expansion_zeroelim(temp8len, temp8, adytail, temp16a);
                finlength = fast_expansion_sum_zeroelim(finlength, finnow, temp16alen, temp16a, finother);
                finswap = finnow; finnow = finother; finother = finswap;
            }
            if (bdytail !== 0) {
                temp8len = scale_expansion_zeroelim(4, aa, -cdxtail, temp8);
                temp16alen = scale_expansion_zeroelim(temp8len, temp8, bdytail, temp16a);
                finlength = fast_expansion_sum_zeroelim(finlength, finnow, temp16alen, temp16a, finother);
                finswap = finnow; finnow = finother; finother = finswap;
            }

            temp32alen = scale_expansion_zeroelim(cxtabtlen, cxtabt, cdxtail, temp32a);
            cxtabttlen = scale_expansion_zeroelim(abttlen, abtt, cdxtail, cxtabtt);
            temp16alen = scale_expansion_zeroelim(cxtabttlen, cxtabtt, 2 * cdx, temp16a);
            temp16blen = scale_expansion_zeroelim(cxtabttlen, cxtabtt, cdxtail, temp16b);
            temp32blen = fast_expansion_sum_zeroelim(temp16alen, temp16a, temp16blen, temp16b, temp32b);
            temp64len = fast_expansion_sum_zeroelim(temp32alen, temp32a, temp32blen, temp32b, temp64);
            finlength = fast_expansion_sum_zeroelim(finlength, finnow, temp64len, temp64, finother);
            finswap = finnow; finnow = finother; finother = finswap;
        }
        if (cdytail !== 0) {
            temp16alen = scale_expansion_zeroelim(cytablen, cytab, cdytail, temp16a);
            cytabtlen = scale_expansion_zeroelim(abtlen, abt$1, cdytail, cytabt);
            temp32alen = scale_expansion_zeroelim(cytabtlen, cytabt, 2 * cdy, temp32a);
            temp48len = fast_expansion_sum_zeroelim(temp16alen, temp16a, temp32alen, temp32a, temp48);
            finlength = fast_expansion_sum_zeroelim(finlength, finnow, temp48len, temp48, finother);
            finswap = finnow; finnow = finother; finother = finswap;

            temp32alen = scale_expansion_zeroelim(cytabtlen, cytabt, cdytail, temp32a);
            cytabttlen = scale_expansion_zeroelim(abttlen, abtt, cdytail, cytabtt);
            temp16alen = scale_expansion_zeroelim(cytabttlen, cytabtt, 2 * cdy, temp16a);
            temp16blen = scale_expansion_zeroelim(cytabttlen, cytabtt, cdytail, temp16b);
            temp32blen = fast_expansion_sum_zeroelim(temp16alen, temp16a, temp16blen, temp16b, temp32b);
            temp64len = fast_expansion_sum_zeroelim(temp32alen, temp32a, temp32blen, temp32b, temp64);
            finlength = fast_expansion_sum_zeroelim(finlength, finnow, temp64len, temp64, finother);
            finswap = finnow; finnow = finother; finother = finswap;
        }
    }

    return finnow[finlength - 1];
}

function incircle(ax, ay, bx, by, cx, cy, dx, dy) {
    const adx = ax - dx;
    const bdx = bx - dx;
    const cdx = cx - dx;
    const ady = ay - dy;
    const bdy = by - dy;
    const cdy = cy - dy;

    const bdxcdy = bdx * cdy;
    const cdxbdy = cdx * bdy;
    const alift = adx * adx + ady * ady;

    const cdxady = cdx * ady;
    const adxcdy = adx * cdy;
    const blift = bdx * bdx + bdy * bdy;

    const adxbdy = adx * bdy;
    const bdxady = bdx * ady;
    const clift = cdx * cdx + cdy * cdy;

    const det =
        alift * (bdxcdy - cdxbdy) +
        blift * (cdxady - adxcdy) +
        clift * (adxbdy - bdxady);

    const permanent =
        (Math.abs(bdxcdy) + Math.abs(cdxbdy)) * alift +
        (Math.abs(cdxady) + Math.abs(adxcdy)) * blift +
        (Math.abs(adxbdy) + Math.abs(bdxady)) * clift;

    const errbound = iccerrboundA * permanent;

    if (det > errbound || -det > errbound) {
        return det;
    }
    return incircleadapt(ax, ay, bx, by, cx, cy, dx, dy, permanent);
}

function incirclefast(ax, ay, bx, by, cx, cy, dx, dy) {
    const adx = ax - dx;
    const ady = ay - dy;
    const bdx = bx - dx;
    const bdy = by - dy;
    const cdx = cx - dx;
    const cdy = cy - dy;

    const abdet = adx * bdy - bdx * ady;
    const bcdet = bdx * cdy - cdx * bdy;
    const cadet = cdx * ady - adx * cdy;
    const alift = adx * adx + ady * ady;
    const blift = bdx * bdx + bdy * bdy;
    const clift = cdx * cdx + cdy * cdy;

    return alift * bcdet + blift * cadet + clift * abdet;
}

const isperrboundA = (16 + 224 * epsilon) * epsilon;
const isperrboundB = (5 + 72 * epsilon) * epsilon;
const isperrboundC = (71 + 1408 * epsilon) * epsilon * epsilon;

const ab$2 = vec(4);
const bc$2 = vec(4);
const cd = vec(4);
const de = vec(4);
const ea = vec(4);
const ac = vec(4);
const bd = vec(4);
const ce = vec(4);
const da = vec(4);
const eb = vec(4);
const temp8a = vec(8);
const temp8b = vec(8);
const temp16 = vec(16);
const abc = vec(24);
const bcd = vec(24);
const cde = vec(24);
const dea = vec(24);
const eab = vec(24);
const abd = vec(24);
const bce = vec(24);
const cda = vec(24);
const deb = vec(24);
const eac = vec(24);
const temp48a = vec(48);
const temp48b = vec(48);
const abcd = vec(96);
const bcde = vec(96);
const cdea = vec(96);
const deab = vec(96);
const eabc = vec(96);
const temp192 = vec(192);
const det384x = vec(384);
const det384y = vec(384);
const det384z = vec(384);
const detxy = vec(768);
const adet$2 = vec(1152);
const bdet$2 = vec(1152);
const cdet$2 = vec(1152);
const ddet = vec(1152);
const edet = vec(1152);
const abdet$2 = vec(2304);
const cddet = vec(2304);
const cdedet = vec(3456);
const deter = vec(5760);

function insphereexact(pa, pb, pc, pd, pe) {
    let axby1, bxcy1, cxdy1, dxey1, exay1;
    let bxay1, cxby1, dxcy1, exdy1, axey1;
    let axcy1, bxdy1, cxey1, dxay1, exby1;
    let cxay1, dxby1, excy1, axdy1, bxey1;
    let axby0, bxcy0, cxdy0, dxey0, exay0;
    let bxay0, cxby0, dxcy0, exdy0, axey0;
    let axcy0, bxdy0, cxey0, dxay0, exby0;
    let cxay0, dxby0, excy0, axdy0, bxey0;
    let temp8alen, temp8blen, temp16len;
    let temp48alen, temp48blen;
    let xlen, ylen, zlen;
    let xylen;
    let i;

    let bvirt, c, ahi, alo, bhi, blo, _i, _j, _0;

    axby1 = pa[0] * pb[1];
    c = splitter * pa[0];
    ahi = c - (c - pa[0]);
    alo = pa[0] - ahi;
    c = splitter * pb[1];
    bhi = c - (c - pb[1]);
    blo = pb[1] - bhi;
    axby0 = alo * blo - (axby1 - ahi * bhi - alo * bhi - ahi * blo);
    bxay1 = pb[0] * pa[1];
    c = splitter * pb[0];
    ahi = c - (c - pb[0]);
    alo = pb[0] - ahi;
    c = splitter * pa[1];
    bhi = c - (c - pa[1]);
    blo = pa[1] - bhi;
    bxay0 = alo * blo - (bxay1 - ahi * bhi - alo * bhi - ahi * blo);
    _i = axby0 - bxay0;
    bvirt = axby0 - _i;
    ab$2[0] = axby0 - (_i + bvirt) + (bvirt - bxay0);
    _j = axby1 + _i;
    bvirt = _j - axby1;
    _0 = axby1 - (_j - bvirt) + (_i - bvirt);
    _i = _0 - bxay1;
    bvirt = _0 - _i;
    ab$2[1] = _0 - (_i + bvirt) + (bvirt - bxay1);
    ab$2[3] = _j + _i;
    bvirt = ab$2[3] - _j;
    ab$2[2] = _j - (ab$2[3] - bvirt) + (_i - bvirt);

    bxcy1 = pb[0] * pc[1];
    c = splitter * pb[0];
    ahi = c - (c - pb[0]);
    alo = pb[0] - ahi;
    c = splitter * pc[1];
    bhi = c - (c - pc[1]);
    blo = pc[1] - bhi;
    bxcy0 = alo * blo - (bxcy1 - ahi * bhi - alo * bhi - ahi * blo);
    cxby1 = pc[0] * pb[1];
    c = splitter * pc[0];
    ahi = c - (c - pc[0]);
    alo = pc[0] - ahi;
    c = splitter * pb[1];
    bhi = c - (c - pb[1]);
    blo = pb[1] - bhi;
    cxby0 = alo * blo - (cxby1 - ahi * bhi - alo * bhi - ahi * blo);
    _i = bxcy0 - cxby0;
    bvirt = bxcy0 - _i;
    bc$2[0] = bxcy0 - (_i + bvirt) + (bvirt - cxby0);
    _j = bxcy1 + _i;
    bvirt = _j - bxcy1;
    _0 = bxcy1 - (_j - bvirt) + (_i - bvirt);
    _i = _0 - cxby1;
    bvirt = _0 - _i;
    bc$2[1] = _0 - (_i + bvirt) + (bvirt - cxby1);
    bc$2[3] = _j + _i;
    bvirt = bc$2[3] - _j;
    bc$2[2] = _j - (bc$2[3] - bvirt) + (_i - bvirt);

    cxdy1 = pc[0] * pd[1];
    c = splitter * pc[0];
    ahi = c - (c - pc[0]);
    alo = pc[0] - ahi;
    c = splitter * pd[1];
    bhi = c - (c - pd[1]);
    blo = pd[1] - bhi;
    cxdy0 = alo * blo - (cxdy1 - ahi * bhi - alo * bhi - ahi * blo);
    dxcy1 = pd[0] * pc[1];
    c = splitter * pd[0];
    ahi = c - (c - pd[0]);
    alo = pd[0] - ahi;
    c = splitter * pc[1];
    bhi = c - (c - pc[1]);
    blo = pc[1] - bhi;
    dxcy0 = alo * blo - (dxcy1 - ahi * bhi - alo * bhi - ahi * blo);
    _i = cxdy0 - dxcy0;
    bvirt = cxdy0 - _i;
    cd[0] = cxdy0 - (_i + bvirt) + (bvirt - dxcy0);
    _j = cxdy1 + _i;
    bvirt = _j - cxdy1;
    _0 = cxdy1 - (_j - bvirt) + (_i - bvirt);
    _i = _0 - dxcy1;
    bvirt = _0 - _i;
    cd[1] = _0 - (_i + bvirt) + (bvirt - dxcy1);
    cd[3] = _j + _i;
    bvirt = cd[3] - _j;
    cd[2] = _j - (cd[3] - bvirt) + (_i - bvirt);

    dxey1 = pd[0] * pe[1];
    c = splitter * pd[0];
    ahi = c - (c - pd[0]);
    alo = pd[0] - ahi;
    c = splitter * pe[1];
    bhi = c - (c - pe[1]);
    blo = pe[1] - bhi;
    dxey0 = alo * blo - (dxey1 - ahi * bhi - alo * bhi - ahi * blo);
    exdy1 = pe[0] * pd[1];
    c = splitter * pe[0];
    ahi = c - (c - pe[0]);
    alo = pe[0] - ahi;
    c = splitter * pd[1];
    bhi = c - (c - pd[1]);
    blo = pd[1] - bhi;
    exdy0 = alo * blo - (exdy1 - ahi * bhi - alo * bhi - ahi * blo);
    _i = dxey0 - exdy0;
    bvirt = dxey0 - _i;
    de[0] = dxey0 - (_i + bvirt) + (bvirt - exdy0);
    _j = dxey1 + _i;
    bvirt = _j - dxey1;
    _0 = dxey1 - (_j - bvirt) + (_i - bvirt);
    _i = _0 - exdy1;
    bvirt = _0 - _i;
    de[1] = _0 - (_i + bvirt) + (bvirt - exdy1);
    de[3] = _j + _i;
    bvirt = de[3] - _j;
    de[2] = _j - (de[3] - bvirt) + (_i - bvirt);

    exay1 = pe[0] * pa[1];
    c = splitter * pe[0];
    ahi = c - (c - pe[0]);
    alo = pe[0] - ahi;
    c = splitter * pa[1];
    bhi = c - (c - pa[1]);
    blo = pa[1] - bhi;
    exay0 = alo * blo - (exay1 - ahi * bhi - alo * bhi - ahi * blo);
    axey1 = pa[0] * pe[1];
    c = splitter * pa[0];
    ahi = c - (c - pa[0]);
    alo = pa[0] - ahi;
    c = splitter * pe[1];
    bhi = c - (c - pe[1]);
    blo = pe[1] - bhi;
    axey0 = alo * blo - (axey1 - ahi * bhi - alo * bhi - ahi * blo);
    _i = exay0 - axey0;
    bvirt = exay0 - _i;
    ea[0] = exay0 - (_i + bvirt) + (bvirt - axey0);
    _j = exay1 + _i;
    bvirt = _j - exay1;
    _0 = exay1 - (_j - bvirt) + (_i - bvirt);
    _i = _0 - axey1;
    bvirt = _0 - _i;
    ea[1] = _0 - (_i + bvirt) + (bvirt - axey1);
    ea[3] = _j + _i;
    bvirt = ea[3] - _j;
    ea[2] = _j - (ea[3] - bvirt) + (_i - bvirt);

    axcy1 = pa[0] * pc[1];
    c = splitter * pa[0];
    ahi = c - (c - pa[0]);
    alo = pa[0] - ahi;
    c = splitter * pc[1];
    bhi = c - (c - pc[1]);
    blo = pc[1] - bhi;
    axcy0 = alo * blo - (axcy1 - ahi * bhi - alo * bhi - ahi * blo);
    cxay1 = pc[0] * pa[1];
    c = splitter * pc[0];
    ahi = c - (c - pc[0]);
    alo = pc[0] - ahi;
    c = splitter * pa[1];
    bhi = c - (c - pa[1]);
    blo = pa[1] - bhi;
    cxay0 = alo * blo - (cxay1 - ahi * bhi - alo * bhi - ahi * blo);
    _i = axcy0 - cxay0;
    bvirt = axcy0 - _i;
    ac[0] = axcy0 - (_i + bvirt) + (bvirt - cxay0);
    _j = axcy1 + _i;
    bvirt = _j - axcy1;
    _0 = axcy1 - (_j - bvirt) + (_i - bvirt);
    _i = _0 - cxay1;
    bvirt = _0 - _i;
    ac[1] = _0 - (_i + bvirt) + (bvirt - cxay1);
    ac[3] = _j + _i;
    bvirt = ac[3] - _j;
    ac[2] = _j - (ac[3] - bvirt) + (_i - bvirt);

    bxdy1 = pb[0] * pd[1];
    c = splitter * pb[0];
    ahi = c - (c - pb[0]);
    alo = pb[0] - ahi;
    c = splitter * pd[1];
    bhi = c - (c - pd[1]);
    blo = pd[1] - bhi;
    bxdy0 = alo * blo - (bxdy1 - ahi * bhi - alo * bhi - ahi * blo);
    dxby1 = pd[0] * pb[1];
    c = splitter * pd[0];
    ahi = c - (c - pd[0]);
    alo = pd[0] - ahi;
    c = splitter * pb[1];
    bhi = c - (c - pb[1]);
    blo = pb[1] - bhi;
    dxby0 = alo * blo - (dxby1 - ahi * bhi - alo * bhi - ahi * blo);
    _i = bxdy0 - dxby0;
    bvirt = bxdy0 - _i;
    bd[0] = bxdy0 - (_i + bvirt) + (bvirt - dxby0);
    _j = bxdy1 + _i;
    bvirt = _j - bxdy1;
    _0 = bxdy1 - (_j - bvirt) + (_i - bvirt);
    _i = _0 - dxby1;
    bvirt = _0 - _i;
    bd[1] = _0 - (_i + bvirt) + (bvirt - dxby1);
    bd[3] = _j + _i;
    bvirt = bd[3] - _j;
    bd[2] = _j - (bd[3] - bvirt) + (_i - bvirt);

    cxey1 = pc[0] * pe[1];
    c = splitter * pc[0];
    ahi = c - (c - pc[0]);
    alo = pc[0] - ahi;
    c = splitter * pe[1];
    bhi = c - (c - pe[1]);
    blo = pe[1] - bhi;
    cxey0 = alo * blo - (cxey1 - ahi * bhi - alo * bhi - ahi * blo);
    excy1 = pe[0] * pc[1];
    c = splitter * pe[0];
    ahi = c - (c - pe[0]);
    alo = pe[0] - ahi;
    c = splitter * pc[1];
    bhi = c - (c - pc[1]);
    blo = pc[1] - bhi;
    excy0 = alo * blo - (excy1 - ahi * bhi - alo * bhi - ahi * blo);
    _i = cxey0 - excy0;
    bvirt = cxey0 - _i;
    ce[0] = cxey0 - (_i + bvirt) + (bvirt - excy0);
    _j = cxey1 + _i;
    bvirt = _j - cxey1;
    _0 = cxey1 - (_j - bvirt) + (_i - bvirt);
    _i = _0 - excy1;
    bvirt = _0 - _i;
    ce[1] = _0 - (_i + bvirt) + (bvirt - excy1);
    ce[3] = _j + _i;
    bvirt = ce[3] - _j;
    ce[2] = _j - (ce[3] - bvirt) + (_i - bvirt);

    dxay1 = pd[0] * pa[1];
    c = splitter * pd[0];
    ahi = c - (c - pd[0]);
    alo = pd[0] - ahi;
    c = splitter * pa[1];
    bhi = c - (c - pa[1]);
    blo = pa[1] - bhi;
    dxay0 = alo * blo - (dxay1 - ahi * bhi - alo * bhi - ahi * blo);
    axdy1 = pa[0] * pd[1];
    c = splitter * pa[0];
    ahi = c - (c - pa[0]);
    alo = pa[0] - ahi;
    c = splitter * pd[1];
    bhi = c - (c - pd[1]);
    blo = pd[1] - bhi;
    axdy0 = alo * blo - (axdy1 - ahi * bhi - alo * bhi - ahi * blo);
    _i = dxay0 - axdy0;
    bvirt = dxay0 - _i;
    da[0] = dxay0 - (_i + bvirt) + (bvirt - axdy0);
    _j = dxay1 + _i;
    bvirt = _j - dxay1;
    _0 = dxay1 - (_j - bvirt) + (_i - bvirt);
    _i = _0 - axdy1;
    bvirt = _0 - _i;
    da[1] = _0 - (_i + bvirt) + (bvirt - axdy1);
    da[3] = _j + _i;
    bvirt = da[3] - _j;
    da[2] = _j - (da[3] - bvirt) + (_i - bvirt);

    exby1 = pe[0] * pb[1];
    c = splitter * pe[0];
    ahi = c - (c - pe[0]);
    alo = pe[0] - ahi;
    c = splitter * pb[1];
    bhi = c - (c - pb[1]);
    blo = pb[1] - bhi;
    exby0 = alo * blo - (exby1 - ahi * bhi - alo * bhi - ahi * blo);
    bxey1 = pb[0] * pe[1];
    c = splitter * pb[0];
    ahi = c - (c - pb[0]);
    alo = pb[0] - ahi;
    c = splitter * pe[1];
    bhi = c - (c - pe[1]);
    blo = pe[1] - bhi;
    bxey0 = alo * blo - (bxey1 - ahi * bhi - alo * bhi - ahi * blo);
    _i = exby0 - bxey0;
    bvirt = exby0 - _i;
    eb[0] = exby0 - (_i + bvirt) + (bvirt - bxey0);
    _j = exby1 + _i;
    bvirt = _j - exby1;
    _0 = exby1 - (_j - bvirt) + (_i - bvirt);
    _i = _0 - bxey1;
    bvirt = _0 - _i;
    eb[1] = _0 - (_i + bvirt) + (bvirt - bxey1);
    eb[3] = _j + _i;
    bvirt = eb[3] - _j;
    eb[2] = _j - (eb[3] - bvirt) + (_i - bvirt);

    temp8alen = scale_expansion_zeroelim(4, bc$2, pa[2], temp8a);
    temp8blen = scale_expansion_zeroelim(4, ac, -pb[2], temp8b);
    temp16len = fast_expansion_sum_zeroelim(temp8alen, temp8a, temp8blen, temp8b, temp16);
    temp8alen = scale_expansion_zeroelim(4, ab$2, pc[2], temp8a);
    const abclen = fast_expansion_sum_zeroelim(temp8alen, temp8a, temp16len, temp16, abc);

    temp8alen = scale_expansion_zeroelim(4, cd, pb[2], temp8a);
    temp8blen = scale_expansion_zeroelim(4, bd, -pc[2], temp8b);
    temp16len = fast_expansion_sum_zeroelim(temp8alen, temp8a, temp8blen, temp8b, temp16);
    temp8alen = scale_expansion_zeroelim(4, bc$2, pd[2], temp8a);
    const bcdlen = fast_expansion_sum_zeroelim(temp8alen, temp8a, temp16len, temp16, bcd);

    temp8alen = scale_expansion_zeroelim(4, de, pc[2], temp8a);
    temp8blen = scale_expansion_zeroelim(4, ce, -pd[2], temp8b);
    temp16len = fast_expansion_sum_zeroelim(temp8alen, temp8a, temp8blen, temp8b, temp16);
    temp8alen = scale_expansion_zeroelim(4, cd, pe[2], temp8a);
    let cdelen = fast_expansion_sum_zeroelim(temp8alen, temp8a, temp16len, temp16, cde);

    temp8alen = scale_expansion_zeroelim(4, ea, pd[2], temp8a);
    temp8blen = scale_expansion_zeroelim(4, da, -pe[2], temp8b);
    temp16len = fast_expansion_sum_zeroelim(temp8alen, temp8a, temp8blen, temp8b, temp16);
    temp8alen = scale_expansion_zeroelim(4, de, pa[2], temp8a);
    const dealen = fast_expansion_sum_zeroelim(temp8alen, temp8a, temp16len, temp16, dea);

    temp8alen = scale_expansion_zeroelim(4, ab$2, pe[2], temp8a);
    temp8blen = scale_expansion_zeroelim(4, eb, -pa[2], temp8b);
    temp16len = fast_expansion_sum_zeroelim(temp8alen, temp8a, temp8blen, temp8b, temp16);
    temp8alen = scale_expansion_zeroelim(4, ea, pb[2], temp8a);
    const eablen = fast_expansion_sum_zeroelim(temp8alen, temp8a, temp16len, temp16, eab);

    temp8alen = scale_expansion_zeroelim(4, bd, pa[2], temp8a);
    temp8blen = scale_expansion_zeroelim(4, da, pb[2], temp8b);
    temp16len = fast_expansion_sum_zeroelim(temp8alen, temp8a, temp8blen, temp8b, temp16);
    temp8alen = scale_expansion_zeroelim(4, ab$2, pd[2], temp8a);
    const abdlen = fast_expansion_sum_zeroelim(temp8alen, temp8a, temp16len, temp16, abd);

    temp8alen = scale_expansion_zeroelim(4, ce, pb[2], temp8a);
    temp8blen = scale_expansion_zeroelim(4, eb, pc[2], temp8b);
    temp16len = fast_expansion_sum_zeroelim(temp8alen, temp8a, temp8blen, temp8b, temp16);
    temp8alen = scale_expansion_zeroelim(4, bc$2, pe[2], temp8a);
    const bcelen = fast_expansion_sum_zeroelim(temp8alen, temp8a, temp16len, temp16, bce);

    temp8alen = scale_expansion_zeroelim(4, da, pc[2], temp8a);
    temp8blen = scale_expansion_zeroelim(4, ac, pd[2], temp8b);
    temp16len = fast_expansion_sum_zeroelim(temp8alen, temp8a, temp8blen, temp8b, temp16);
    temp8alen = scale_expansion_zeroelim(4, cd, pa[2], temp8a);
    const cdalen = fast_expansion_sum_zeroelim(temp8alen, temp8a, temp16len, temp16, cda);

    temp8alen = scale_expansion_zeroelim(4, eb, pd[2], temp8a);
    temp8blen = scale_expansion_zeroelim(4, bd, pe[2], temp8b);
    temp16len = fast_expansion_sum_zeroelim(temp8alen, temp8a, temp8blen, temp8b, temp16);
    temp8alen = scale_expansion_zeroelim(4, de, pb[2], temp8a);
    const deblen = fast_expansion_sum_zeroelim(temp8alen, temp8a, temp16len, temp16, deb);

    temp8alen = scale_expansion_zeroelim(4, ac, pe[2], temp8a);
    temp8blen = scale_expansion_zeroelim(4, ce, pa[2], temp8b);
    temp16len = fast_expansion_sum_zeroelim(temp8alen, temp8a, temp8blen, temp8b, temp16);
    temp8alen = scale_expansion_zeroelim(4, ea, pc[2], temp8a);
    const eaclen = fast_expansion_sum_zeroelim(temp8alen, temp8a, temp16len, temp16, eac);

    temp48alen = fast_expansion_sum_zeroelim(cdelen, cde, bcelen, bce, temp48a);
    temp48blen = fast_expansion_sum_zeroelim(deblen, deb, bcdlen, bcd, temp48b);
    for (i = 0; i < temp48blen; i++) {
        temp48b[i] = -temp48b[i];
    }
    const bcdelen = fast_expansion_sum_zeroelim(temp48alen, temp48a, temp48blen, temp48b, bcde);
    xlen = scale_expansion_zeroelim(bcdelen, bcde, pa[0], temp192);
    xlen = scale_expansion_zeroelim(xlen, temp192, pa[0], det384x);
    ylen = scale_expansion_zeroelim(bcdelen, bcde, pa[1], temp192);
    ylen = scale_expansion_zeroelim(ylen, temp192, pa[1], det384y);
    zlen = scale_expansion_zeroelim(bcdelen, bcde, pa[2], temp192);
    zlen = scale_expansion_zeroelim(zlen, temp192, pa[2], det384z);
    xylen = fast_expansion_sum_zeroelim(xlen, det384x, ylen, det384y, detxy);
    const alen = fast_expansion_sum_zeroelim(xylen, detxy, zlen, det384z, adet$2);

    temp48alen = fast_expansion_sum_zeroelim(dealen, dea, cdalen, cda, temp48a);
    temp48blen = fast_expansion_sum_zeroelim(eaclen, eac, cdelen, cde, temp48b);
    for (i = 0; i < temp48blen; i++) {
        temp48b[i] = -temp48b[i];
    }
    const cdealen = fast_expansion_sum_zeroelim(temp48alen, temp48a, temp48blen, temp48b, cdea);
    xlen = scale_expansion_zeroelim(cdealen, cdea, pb[0], temp192);
    xlen = scale_expansion_zeroelim(xlen, temp192, pb[0], det384x);
    ylen = scale_expansion_zeroelim(cdealen, cdea, pb[1], temp192);
    ylen = scale_expansion_zeroelim(ylen, temp192, pb[1], det384y);
    zlen = scale_expansion_zeroelim(cdealen, cdea, pb[2], temp192);
    zlen = scale_expansion_zeroelim(zlen, temp192, pb[2], det384z);
    xylen = fast_expansion_sum_zeroelim(xlen, det384x, ylen, det384y, detxy);
    const blen = fast_expansion_sum_zeroelim(xylen, detxy, zlen, det384z, bdet$2);

    temp48alen = fast_expansion_sum_zeroelim(eablen, eab, deblen, deb, temp48a);
    temp48blen = fast_expansion_sum_zeroelim(abdlen, abd, dealen, dea, temp48b);
    for (i = 0; i < temp48blen; i++) {
        temp48b[i] = -temp48b[i];
    }
    const deablen = fast_expansion_sum_zeroelim(temp48alen, temp48a, temp48blen, temp48b, deab);
    xlen = scale_expansion_zeroelim(deablen, deab, pc[0], temp192);
    xlen = scale_expansion_zeroelim(xlen, temp192, pc[0], det384x);
    ylen = scale_expansion_zeroelim(deablen, deab, pc[1], temp192);
    ylen = scale_expansion_zeroelim(ylen, temp192, pc[1], det384y);
    zlen = scale_expansion_zeroelim(deablen, deab, pc[2], temp192);
    zlen = scale_expansion_zeroelim(zlen, temp192, pc[2], det384z);
    xylen = fast_expansion_sum_zeroelim(xlen, det384x, ylen, det384y, detxy);
    const clen = fast_expansion_sum_zeroelim(xylen, detxy, zlen, det384z, cdet$2);

    temp48alen = fast_expansion_sum_zeroelim(abclen, abc, eaclen, eac, temp48a);
    temp48blen = fast_expansion_sum_zeroelim(bcelen, bce, eablen, eab, temp48b);
    for (i = 0; i < temp48blen; i++) {
        temp48b[i] = -temp48b[i];
    }
    const eabclen = fast_expansion_sum_zeroelim(temp48alen, temp48a, temp48blen, temp48b, eabc);
    xlen = scale_expansion_zeroelim(eabclen, eabc, pd[0], temp192);
    xlen = scale_expansion_zeroelim(xlen, temp192, pd[0], det384x);
    ylen = scale_expansion_zeroelim(eabclen, eabc, pd[1], temp192);
    ylen = scale_expansion_zeroelim(ylen, temp192, pd[1], det384y);
    zlen = scale_expansion_zeroelim(eabclen, eabc, pd[2], temp192);
    zlen = scale_expansion_zeroelim(zlen, temp192, pd[2], det384z);
    xylen = fast_expansion_sum_zeroelim(xlen, det384x, ylen, det384y, detxy);
    const dlen = fast_expansion_sum_zeroelim(xylen, detxy, zlen, det384z, ddet);

    temp48alen = fast_expansion_sum_zeroelim(bcdlen, bcd, abdlen, abd, temp48a);
    temp48blen = fast_expansion_sum_zeroelim(cdalen, cda, abclen, abc, temp48b);
    for (i = 0; i < temp48blen; i++) {
        temp48b[i] = -temp48b[i];
    }
    const abcdlen = fast_expansion_sum_zeroelim(temp48alen, temp48a, temp48blen, temp48b, abcd);
    xlen = scale_expansion_zeroelim(abcdlen, abcd, pe[0], temp192);
    xlen = scale_expansion_zeroelim(xlen, temp192, pe[0], det384x);
    ylen = scale_expansion_zeroelim(abcdlen, abcd, pe[1], temp192);
    ylen = scale_expansion_zeroelim(ylen, temp192, pe[1], det384y);
    zlen = scale_expansion_zeroelim(abcdlen, abcd, pe[2], temp192);
    zlen = scale_expansion_zeroelim(zlen, temp192, pe[2], det384z);
    xylen = fast_expansion_sum_zeroelim(xlen, det384x, ylen, det384y, detxy);
    const elen = fast_expansion_sum_zeroelim(xylen, detxy, zlen, det384z, edet);

    const ablen = fast_expansion_sum_zeroelim(alen, adet$2, blen, bdet$2, abdet$2);
    const cdlen = fast_expansion_sum_zeroelim(clen, cdet$2, dlen, ddet, cddet);
    cdelen = fast_expansion_sum_zeroelim(cdlen, cddet, elen, edet, cdedet);
    const deterlen = fast_expansion_sum_zeroelim(ablen, abdet$2, cdelen, cdedet, deter);

    return deter[deterlen - 1];
}

const temp8c = vec(8);
const temp24 = vec(24);
const temp48$1 = vec(48);
const xdet = vec(96);
const ydet = vec(96);
const zdet = vec(96);
const xydet = vec(192);
const fin1$2 = vec(1152);

function insphereadapt(ax, ay, az, bx, by, bz, cx, cy, cz, dx, dy, dz, ex, ey, ez, permanent) {
    let aexbey1, bexaey1, bexcey1, cexbey1;
    let cexdey1, dexcey1, dexaey1, aexdey1;
    let aexcey1, cexaey1, bexdey1, dexbey1;
    let aexbey0, bexaey0, bexcey0, cexbey0;
    let cexdey0, dexcey0, dexaey0, aexdey0;
    let aexcey0, cexaey0, bexdey0, dexbey0;
    let ab3, bc3, cd3, da3, ac3, bd3;
    let temp8alen, temp8blen, temp8clen, temp16len, temp24len, temp48len;
    let xlen, ylen, zlen, xylen;

    let aextail, bextail, cextail, dextail;
    let aeytail, beytail, ceytail, deytail;
    let aeztail, beztail, ceztail, deztail;

    let bvirt, c, ahi, alo, bhi, blo, _i, _j, _0;

    const aex = ax - ex;
    const bex = bx - ex;
    const cex = cx - ex;
    const dex = dx - ex;
    const aey = ay - ey;
    const bey = by - ey;
    const cey = cy - ey;
    const dey = dy - ey;
    const aez = az - ez;
    const bez = bz - ez;
    const cez = cz - ez;
    const dez = dz - ez;

    aexbey1 = aex * bey;
    c = splitter * aex;
    ahi = c - (c - aex);
    alo = aex - ahi;
    c = splitter * bey;
    bhi = c - (c - bey);
    blo = bey - bhi;
    aexbey0 = alo * blo - (aexbey1 - ahi * bhi - alo * bhi - ahi * blo);
    bexaey1 = bex * aey;
    c = splitter * bex;
    ahi = c - (c - bex);
    alo = bex - ahi;
    c = splitter * aey;
    bhi = c - (c - aey);
    blo = aey - bhi;
    bexaey0 = alo * blo - (bexaey1 - ahi * bhi - alo * bhi - ahi * blo);
    _i = aexbey0 - bexaey0;
    bvirt = aexbey0 - _i;
    ab$2[0] = aexbey0 - (_i + bvirt) + (bvirt - bexaey0);
    _j = aexbey1 + _i;
    bvirt = _j - aexbey1;
    _0 = aexbey1 - (_j - bvirt) + (_i - bvirt);
    _i = _0 - bexaey1;
    bvirt = _0 - _i;
    ab$2[1] = _0 - (_i + bvirt) + (bvirt - bexaey1);
    ab3 = _j + _i;
    bvirt = ab3 - _j;
    ab$2[2] = _j - (ab3 - bvirt) + (_i - bvirt);
    ab$2[3] = ab3;

    bexcey1 = bex * cey;
    c = splitter * bex;
    ahi = c - (c - bex);
    alo = bex - ahi;
    c = splitter * cey;
    bhi = c - (c - cey);
    blo = cey - bhi;
    bexcey0 = alo * blo - (bexcey1 - ahi * bhi - alo * bhi - ahi * blo);
    cexbey1 = cex * bey;
    c = splitter * cex;
    ahi = c - (c - cex);
    alo = cex - ahi;
    c = splitter * bey;
    bhi = c - (c - bey);
    blo = bey - bhi;
    cexbey0 = alo * blo - (cexbey1 - ahi * bhi - alo * bhi - ahi * blo);
    _i = bexcey0 - cexbey0;
    bvirt = bexcey0 - _i;
    bc$2[0] = bexcey0 - (_i + bvirt) + (bvirt - cexbey0);
    _j = bexcey1 + _i;
    bvirt = _j - bexcey1;
    _0 = bexcey1 - (_j - bvirt) + (_i - bvirt);
    _i = _0 - cexbey1;
    bvirt = _0 - _i;
    bc$2[1] = _0 - (_i + bvirt) + (bvirt - cexbey1);
    bc3 = _j + _i;
    bvirt = bc3 - _j;
    bc$2[2] = _j - (bc3 - bvirt) + (_i - bvirt);
    bc$2[3] = bc3;

    cexdey1 = cex * dey;
    c = splitter * cex;
    ahi = c - (c - cex);
    alo = cex - ahi;
    c = splitter * dey;
    bhi = c - (c - dey);
    blo = dey - bhi;
    cexdey0 = alo * blo - (cexdey1 - ahi * bhi - alo * bhi - ahi * blo);
    dexcey1 = dex * cey;
    c = splitter * dex;
    ahi = c - (c - dex);
    alo = dex - ahi;
    c = splitter * cey;
    bhi = c - (c - cey);
    blo = cey - bhi;
    dexcey0 = alo * blo - (dexcey1 - ahi * bhi - alo * bhi - ahi * blo);
    _i = cexdey0 - dexcey0;
    bvirt = cexdey0 - _i;
    cd[0] = cexdey0 - (_i + bvirt) + (bvirt - dexcey0);
    _j = cexdey1 + _i;
    bvirt = _j - cexdey1;
    _0 = cexdey1 - (_j - bvirt) + (_i - bvirt);
    _i = _0 - dexcey1;
    bvirt = _0 - _i;
    cd[1] = _0 - (_i + bvirt) + (bvirt - dexcey1);
    cd3 = _j + _i;
    bvirt = cd3 - _j;
    cd[2] = _j - (cd3 - bvirt) + (_i - bvirt);
    cd[3] = cd3;

    dexaey1 = dex * aey;
    c = splitter * dex;
    ahi = c - (c - dex);
    alo = dex - ahi;
    c = splitter * aey;
    bhi = c - (c - aey);
    blo = aey - bhi;
    dexaey0 = alo * blo - (dexaey1 - ahi * bhi - alo * bhi - ahi * blo);
    aexdey1 = aex * dey;
    c = splitter * aex;
    ahi = c - (c - aex);
    alo = aex - ahi;
    c = splitter * dey;
    bhi = c - (c - dey);
    blo = dey - bhi;
    aexdey0 = alo * blo - (aexdey1 - ahi * bhi - alo * bhi - ahi * blo);
    _i = dexaey0 - aexdey0;
    bvirt = dexaey0 - _i;
    da[0] = dexaey0 - (_i + bvirt) + (bvirt - aexdey0);
    _j = dexaey1 + _i;
    bvirt = _j - dexaey1;
    _0 = dexaey1 - (_j - bvirt) + (_i - bvirt);
    _i = _0 - aexdey1;
    bvirt = _0 - _i;
    da[1] = _0 - (_i + bvirt) + (bvirt - aexdey1);
    da3 = _j + _i;
    bvirt = da3 - _j;
    da[2] = _j - (da3 - bvirt) + (_i - bvirt);
    da[3] = da3;

    aexcey1 = aex * cey;
    c = splitter * aex;
    ahi = c - (c - aex);
    alo = aex - ahi;
    c = splitter * cey;
    bhi = c - (c - cey);
    blo = cey - bhi;
    aexcey0 = alo * blo - (aexcey1 - ahi * bhi - alo * bhi - ahi * blo);
    cexaey1 = cex * aey;
    c = splitter * cex;
    ahi = c - (c - cex);
    alo = cex - ahi;
    c = splitter * aey;
    bhi = c - (c - aey);
    blo = aey - bhi;
    cexaey0 = alo * blo - (cexaey1 - ahi * bhi - alo * bhi - ahi * blo);
    _i = aexcey0 - cexaey0;
    bvirt = aexcey0 - _i;
    ac[0] = aexcey0 - (_i + bvirt) + (bvirt - cexaey0);
    _j = aexcey1 + _i;
    bvirt = _j - aexcey1;
    _0 = aexcey1 - (_j - bvirt) + (_i - bvirt);
    _i = _0 - cexaey1;
    bvirt = _0 - _i;
    ac[1] = _0 - (_i + bvirt) + (bvirt - cexaey1);
    ac3 = _j + _i;
    bvirt = ac3 - _j;
    ac[2] = _j - (ac3 - bvirt) + (_i - bvirt);
    ac[3] = ac3;

    bexdey1 = bex * dey;
    c = splitter * bex;
    ahi = c - (c - bex);
    alo = bex - ahi;
    c = splitter * dey;
    bhi = c - (c - dey);
    blo = dey - bhi;
    bexdey0 = alo * blo - (bexdey1 - ahi * bhi - alo * bhi - ahi * blo);
    dexbey1 = dex * bey;
    c = splitter * dex;
    ahi = c - (c - dex);
    alo = dex - ahi;
    c = splitter * bey;
    bhi = c - (c - bey);
    blo = bey - bhi;
    dexbey0 = alo * blo - (dexbey1 - ahi * bhi - alo * bhi - ahi * blo);
    _i = bexdey0 - dexbey0;
    bvirt = bexdey0 - _i;
    bd[0] = bexdey0 - (_i + bvirt) + (bvirt - dexbey0);
    _j = bexdey1 + _i;
    bvirt = _j - bexdey1;
    _0 = bexdey1 - (_j - bvirt) + (_i - bvirt);
    _i = _0 - dexbey1;
    bvirt = _0 - _i;
    bd[1] = _0 - (_i + bvirt) + (bvirt - dexbey1);
    bd3 = _j + _i;
    bvirt = bd3 - _j;
    bd[2] = _j - (bd3 - bvirt) + (_i - bvirt);
    bd[3] = bd3;

    temp8alen = scale_expansion_zeroelim(4, cd, bez, temp8a);
    temp8blen = scale_expansion_zeroelim(4, bd, -cez, temp8b);
    temp8clen = scale_expansion_zeroelim(4, bc$2, dez, temp8c);
    temp16len = fast_expansion_sum_zeroelim(temp8alen, temp8a, temp8blen, temp8b, temp16);
    temp24len = fast_expansion_sum_zeroelim(temp8clen, temp8c, temp16len, temp16, temp24);
    temp48len = scale_expansion_zeroelim(temp24len, temp24, aex, temp48$1);
    xlen = scale_expansion_zeroelim(temp48len, temp48$1, -aex, xdet);
    temp48len = scale_expansion_zeroelim(temp24len, temp24, aey, temp48$1);
    ylen = scale_expansion_zeroelim(temp48len, temp48$1, -aey, ydet);
    temp48len = scale_expansion_zeroelim(temp24len, temp24, aez, temp48$1);
    zlen = scale_expansion_zeroelim(temp48len, temp48$1, -aez, zdet);
    xylen = fast_expansion_sum_zeroelim(xlen, xdet, ylen, ydet, xydet);
    const alen = fast_expansion_sum_zeroelim(xylen, xydet, zlen, zdet, adet$2);

    temp8alen = scale_expansion_zeroelim(4, da, cez, temp8a);
    temp8blen = scale_expansion_zeroelim(4, ac, dez, temp8b);
    temp8clen = scale_expansion_zeroelim(4, cd, aez, temp8c);
    temp16len = fast_expansion_sum_zeroelim(temp8alen, temp8a, temp8blen, temp8b, temp16);
    temp24len = fast_expansion_sum_zeroelim(temp8clen, temp8c, temp16len, temp16, temp24);
    temp48len = scale_expansion_zeroelim(temp24len, temp24, bex, temp48$1);
    xlen = scale_expansion_zeroelim(temp48len, temp48$1, bex, xdet);
    temp48len = scale_expansion_zeroelim(temp24len, temp24, bey, temp48$1);
    ylen = scale_expansion_zeroelim(temp48len, temp48$1, bey, ydet);
    temp48len = scale_expansion_zeroelim(temp24len, temp24, bez, temp48$1);
    zlen = scale_expansion_zeroelim(temp48len, temp48$1, bez, zdet);
    xylen = fast_expansion_sum_zeroelim(xlen, xdet, ylen, ydet, xydet);
    const blen = fast_expansion_sum_zeroelim(xylen, xydet, zlen, zdet, bdet$2);

    temp8alen = scale_expansion_zeroelim(4, ab$2, dez, temp8a);
    temp8blen = scale_expansion_zeroelim(4, bd, aez, temp8b);
    temp8clen = scale_expansion_zeroelim(4, da, bez, temp8c);
    temp16len = fast_expansion_sum_zeroelim(temp8alen, temp8a, temp8blen, temp8b, temp16);
    temp24len = fast_expansion_sum_zeroelim(temp8clen, temp8c, temp16len, temp16, temp24);
    temp48len = scale_expansion_zeroelim(temp24len, temp24, cex, temp48$1);
    xlen = scale_expansion_zeroelim(temp48len, temp48$1, -cex, xdet);
    temp48len = scale_expansion_zeroelim(temp24len, temp24, cey, temp48$1);
    ylen = scale_expansion_zeroelim(temp48len, temp48$1, -cey, ydet);
    temp48len = scale_expansion_zeroelim(temp24len, temp24, cez, temp48$1);
    zlen = scale_expansion_zeroelim(temp48len, temp48$1, -cez, zdet);
    xylen = fast_expansion_sum_zeroelim(xlen, xdet, ylen, ydet, xydet);
    const clen = fast_expansion_sum_zeroelim(xylen, xydet, zlen, zdet, cdet$2);

    temp8alen = scale_expansion_zeroelim(4, bc$2, aez, temp8a);
    temp8blen = scale_expansion_zeroelim(4, ac, -bez, temp8b);
    temp8clen = scale_expansion_zeroelim(4, ab$2, cez, temp8c);
    temp16len = fast_expansion_sum_zeroelim(temp8alen, temp8a, temp8blen, temp8b, temp16);
    temp24len = fast_expansion_sum_zeroelim(temp8clen, temp8c, temp16len, temp16, temp24);
    temp48len = scale_expansion_zeroelim(temp24len, temp24, dex, temp48$1);
    xlen = scale_expansion_zeroelim(temp48len, temp48$1, dex, xdet);
    temp48len = scale_expansion_zeroelim(temp24len, temp24, dey, temp48$1);
    ylen = scale_expansion_zeroelim(temp48len, temp48$1, dey, ydet);
    temp48len = scale_expansion_zeroelim(temp24len, temp24, dez, temp48$1);
    zlen = scale_expansion_zeroelim(temp48len, temp48$1, dez, zdet);
    xylen = fast_expansion_sum_zeroelim(xlen, xdet, ylen, ydet, xydet);
    const dlen = fast_expansion_sum_zeroelim(xylen, xydet, zlen, zdet, ddet);

    const ablen = fast_expansion_sum_zeroelim(alen, adet$2, blen, bdet$2, abdet$2);
    const cdlen = fast_expansion_sum_zeroelim(clen, cdet$2, dlen, ddet, cddet);
    const finlength = fast_expansion_sum_zeroelim(ablen, abdet$2, cdlen, cddet, fin1$2);

    let det = estimate(finlength, fin1$2);
    let errbound = isperrboundB * permanent;
    if (det >= errbound || -det >= errbound) {
        return det;
    }

    bvirt = ax - aex;
    aextail = ax - (aex + bvirt) + (bvirt - ex);
    bvirt = ay - aey;
    aeytail = ay - (aey + bvirt) + (bvirt - ey);
    bvirt = az - aez;
    aeztail = az - (aez + bvirt) + (bvirt - ez);
    bvirt = bx - bex;
    bextail = bx - (bex + bvirt) + (bvirt - ex);
    bvirt = by - bey;
    beytail = by - (bey + bvirt) + (bvirt - ey);
    bvirt = bz - bez;
    beztail = bz - (bez + bvirt) + (bvirt - ez);
    bvirt = cx - cex;
    cextail = cx - (cex + bvirt) + (bvirt - ex);
    bvirt = cy - cey;
    ceytail = cy - (cey + bvirt) + (bvirt - ey);
    bvirt = cz - cez;
    ceztail = cz - (cez + bvirt) + (bvirt - ez);
    bvirt = dx - dex;
    dextail = dx - (dex + bvirt) + (bvirt - ex);
    bvirt = dy - dey;
    deytail = dy - (dey + bvirt) + (bvirt - ey);
    bvirt = dz - dez;
    deztail = dz - (dez + bvirt) + (bvirt - ez);
    if (aextail === 0 && aeytail === 0 && aeztail === 0 &&
        bextail === 0 && beytail === 0 && beztail === 0 &&
        cextail === 0 && ceytail === 0 && ceztail === 0 &&
        dextail === 0 && deytail === 0 && deztail === 0) {
        return det;
    }

    errbound = isperrboundC * permanent + resulterrbound * Math.abs(det);

    const abeps = (aex * beytail + bey * aextail) - (aey * bextail + bex * aeytail);
    const bceps = (bex * ceytail + cey * bextail) - (bey * cextail + cex * beytail);
    const cdeps = (cex * deytail + dey * cextail) - (cey * dextail + dex * ceytail);
    const daeps = (dex * aeytail + aey * dextail) - (dey * aextail + aex * deytail);
    const aceps = (aex * ceytail + cey * aextail) - (aey * cextail + cex * aeytail);
    const bdeps = (bex * deytail + dey * bextail) - (bey * dextail + dex * beytail);
    det +=
        (((bex * bex + bey * bey + bez * bez) * ((cez * daeps + dez * aceps + aez * cdeps) +
        (ceztail * da3 + deztail * ac3 + aeztail * cd3)) + (dex * dex + dey * dey + dez * dez) *
        ((aez * bceps - bez * aceps + cez * abeps) + (aeztail * bc3 - beztail * ac3 + ceztail * ab3))) -
        ((aex * aex + aey * aey + aez * aez) * ((bez * cdeps - cez * bdeps + dez * bceps) +
        (beztail * cd3 - ceztail * bd3 + deztail * bc3)) + (cex * cex + cey * cey + cez * cez) *
        ((dez * abeps + aez * bdeps + bez * daeps) + (deztail * ab3 + aeztail * bd3 + beztail * da3)))) +
        2 * (((bex * bextail + bey * beytail + bez * beztail) * (cez * da3 + dez * ac3 + aez * cd3) +
        (dex * dextail + dey * deytail + dez * deztail) * (aez * bc3 - bez * ac3 + cez * ab3)) -
        ((aex * aextail + aey * aeytail + aez * aeztail) * (bez * cd3 - cez * bd3 + dez * bc3) +
        (cex * cextail + cey * ceytail + cez * ceztail) * (dez * ab3 + aez * bd3 + bez * da3)));

    if (det >= errbound || -det >= errbound) {
        return det;
    }

    return insphereexact(ax, ay, az, bx, by);
}

function insphere(ax, ay, az, bx, by, bz, cx, cy, cz, dx, dy, dz, ex, ey, ez) {
    const aex = ax - ex;
    const bex = bx - ex;
    const cex = cx - ex;
    const dex = dx - ex;
    const aey = ay - ey;
    const bey = by - ey;
    const cey = cy - ey;
    const dey = dy - ey;
    const aez = az - ez;
    const bez = bz - ez;
    const cez = cz - ez;
    const dez = dz - ez;

    const aexbey = aex * bey;
    const bexaey = bex * aey;
    const ab = aexbey - bexaey;
    const bexcey = bex * cey;
    const cexbey = cex * bey;
    const bc = bexcey - cexbey;
    const cexdey = cex * dey;
    const dexcey = dex * cey;
    const cd = cexdey - dexcey;
    const dexaey = dex * aey;
    const aexdey = aex * dey;
    const da = dexaey - aexdey;

    const aexcey = aex * cey;
    const cexaey = cex * aey;
    const ac = aexcey - cexaey;
    const bexdey = bex * dey;
    const dexbey = dex * bey;
    const bd = bexdey - dexbey;

    const abc = aez * bc - bez * ac + cez * ab;
    const bcd = bez * cd - cez * bd + dez * bc;
    const cda = cez * da + dez * ac + aez * cd;
    const dab = dez * ab + aez * bd + bez * da;

    const alift = aex * aex + aey * aey + aez * aez;
    const blift = bex * bex + bey * bey + bez * bez;
    const clift = cex * cex + cey * cey + cez * cez;
    const dlift = dex * dex + dey * dey + dez * dez;

    const det = (clift * dab - dlift * abc) + (alift * bcd - blift * cda);

    const aezplus = Math.abs(aez);
    const bezplus = Math.abs(bez);
    const cezplus = Math.abs(cez);
    const dezplus = Math.abs(dez);
    const aexbeyplus = Math.abs(aexbey);
    const bexaeyplus = Math.abs(bexaey);
    const bexceyplus = Math.abs(bexcey);
    const cexbeyplus = Math.abs(cexbey);
    const cexdeyplus = Math.abs(cexdey);
    const dexceyplus = Math.abs(dexcey);
    const dexaeyplus = Math.abs(dexaey);
    const aexdeyplus = Math.abs(aexdey);
    const aexceyplus = Math.abs(aexcey);
    const cexaeyplus = Math.abs(cexaey);
    const bexdeyplus = Math.abs(bexdey);
    const dexbeyplus = Math.abs(dexbey);
    const permanent =
        ((cexdeyplus + dexceyplus) * bezplus + (dexbeyplus + bexdeyplus) * cezplus + (bexceyplus + cexbeyplus) * dezplus) * alift +
        ((dexaeyplus + aexdeyplus) * cezplus + (aexceyplus + cexaeyplus) * dezplus + (cexdeyplus + dexceyplus) * aezplus) * blift +
        ((aexbeyplus + bexaeyplus) * dezplus + (bexdeyplus + dexbeyplus) * aezplus + (dexaeyplus + aexdeyplus) * bezplus) * clift +
        ((bexceyplus + cexbeyplus) * aezplus + (cexaeyplus + aexceyplus) * bezplus + (aexbeyplus + bexaeyplus) * cezplus) * dlift;

    const errbound = isperrboundA * permanent;
    if (det > errbound || -det > errbound) {
        return det;
    }
    return -insphereadapt(ax, ay, az, bx, by, bz, cx, cy, cz, dx, dy, dz, ex, ey, ez, permanent);
}

function inspherefast(pax, pay, paz, pbx, pby, pbz, pcx, pcy, pcz, pdx, pdy, pdz, pex, pey, pez) {
    const aex = pax - pex;
    const bex = pbx - pex;
    const cex = pcx - pex;
    const dex = pdx - pex;
    const aey = pay - pey;
    const bey = pby - pey;
    const cey = pcy - pey;
    const dey = pdy - pey;
    const aez = paz - pez;
    const bez = pbz - pez;
    const cez = pcz - pez;
    const dez = pdz - pez;

    const ab = aex * bey - bex * aey;
    const bc = bex * cey - cex * bey;
    const cd = cex * dey - dex * cey;
    const da = dex * aey - aex * dey;

    const ac = aex * cey - cex * aey;
    const bd = bex * dey - dex * bey;

    const abc = aez * bc - bez * ac + cez * ab;
    const bcd = bez * cd - cez * bd + dez * bc;
    const cda = cez * da + dez * ac + aez * cd;
    const dab = dez * ab + aez * bd + bez * da;

    const alift = aex * aex + aey * aey + aez * aez;
    const blift = bex * bex + bey * bey + bez * bez;
    const clift = cex * cex + cey * cey + cez * cez;
    const dlift = dex * dex + dey * dey + dez * dez;

    return (clift * dab - dlift * abc) + (alift * bcd - blift * cda);
}

exports.incircle = incircle;
exports.incirclefast = incirclefast;
exports.insphere = insphere;
exports.inspherefast = inspherefast;
exports.orient2d = orient2d;
exports.orient2dfast = orient2dfast;
exports.orient3d = orient3d;
exports.orient3dfast = orient3dfast;

Object.defineProperty(exports, '__esModule', { value: true });

}));
